# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AgentContainerOperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the container operation for a specific version of an agent."""

    NOT_STARTED = "NotStarted"
    """The container operation is not started."""
    IN_PROGRESS = "InProgress"
    """The container operation is in progress."""
    SUCCEEDED = "Succeeded"
    """The container operation has succeeded."""
    FAILED = "Failed"
    """The container operation has failed."""


class AgentContainerStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the container of a specific version of an agent."""

    STARTING = "Starting"
    """The container is starting."""
    RUNNING = "Running"
    """The container is running."""
    STOPPING = "Stopping"
    """The container is stopping."""
    STOPPED = "Stopped"
    """The container is stopped."""
    FAILED = "Failed"
    """The container has failed."""
    DELETING = "Deleting"
    """The container is deleting."""
    DELETED = "Deleted"
    """The container is deleted."""
    UPDATING = "Updating"
    """The container is updating."""


class AgentKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of AgentKind."""

    PROMPT = "prompt"
    HOSTED = "hosted"
    CONTAINER_APP = "container_app"
    WORKFLOW = "workflow"


class AgentProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of AgentProtocol."""

    ACTIVITY_PROTOCOL = "activity_protocol"
    RESPONSES = "responses"


class AnnotationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of AnnotationType."""

    FILE_CITATION = "file_citation"
    URL_CITATION = "url_citation"
    FILE_PATH = "file_path"
    CONTAINER_FILE_CITATION = "container_file_citation"


class AttackStrategy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Strategies for attacks."""

    EASY = "easy"
    """Represents a default set of easy complexity attacks. Easy complexity attacks require less
    effort, such as translation of a prompt into some encoding, and does not require any Large
    Language Model to convert or orchestrate."""
    MODERATE = "moderate"
    """Represents a default set of moderate complexity attacks. Moderate complexity attacks require
    having access to resources such as another generative AI model."""
    DIFFICULT = "difficult"
    """Represents a default set of difficult complexity attacks. Difficult complexity attacks include
    attacks that require access to significant resources and effort to execute an attack such as
    knowledge of search-based algorithms in addition to a generative AI model."""
    ASCII_ART = "ascii_art"
    """Generates visual art using ASCII characters, often used for creative or obfuscation purposes."""
    ASCII_SMUGGLER = "ascii_smuggler"
    """Conceals data within ASCII characters, making it harder to detect."""
    ATBASH = "atbash"
    """Implements the Atbash cipher, a simple substitution cipher where each letter is mapped to its
    reverse."""
    BASE64 = "base64"
    """Encodes binary data into a text format using Base64, commonly used for data transmission."""
    BINARY = "binary"
    """Converts text into binary code, representing data in a series of 0s and 1s."""
    CAESAR = "caesar"
    """Applies the Caesar cipher, a substitution cipher that shifts characters by a fixed number of
    positions."""
    CHARACTER_SPACE = "character_space"
    """Alters text by adding spaces between characters, often used for obfuscation."""
    JAILBREAK = "jailbreak"
    """Injects specially crafted prompts to bypass AI safeguards, known as User Injected Prompt
    Attacks (UPIA)."""
    ANSII_ATTACK = "ansii_attack"
    """Utilizes ANSI escape sequences to manipulate text appearance and behavior."""
    CHARACTER_SWAP = "character_swap"
    """Swaps characters within text to create variations or obfuscate the original content."""
    SUFFIX_APPEND = "suffix_append"
    """Appends an adversarial suffix to the prompt."""
    STRING_JOIN = "string_join"
    """Joins multiple strings together, often used for concatenation or obfuscation."""
    UNICODE_CONFUSABLE = "unicode_confusable"
    """Uses Unicode characters that look similar to standard characters, creating visual confusion."""
    UNICODE_SUBSTITUTION = "unicode_substitution"
    """Substitutes standard characters with Unicode equivalents, often for obfuscation."""
    DIACRITIC = "diacritic"
    """Adds diacritical marks to characters, changing their appearance and sometimes their meaning."""
    FLIP = "flip"
    """Flips characters from front to back, creating a mirrored effect."""
    LEETSPEAK = "leetspeak"
    """Transforms text into Leetspeak, a form of encoding that replaces letters with similar-looking
    numbers or symbols."""
    ROT13 = "rot13"
    """Applies the ROT13 cipher, a simple substitution cipher that shifts characters by 13 positions."""
    MORSE = "morse"
    """Encodes text into Morse code, using dots and dashes to represent characters."""
    URL = "url"
    """Encodes text into URL format."""
    BASELINE = "baseline"
    """Represents the baseline direct adversarial probing, which is used by attack strategies as the
    attack objective."""


class AzureAISearchQueryType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Available query types for Azure AI Search tool."""

    SIMPLE = "simple"
    """Query type ``simple``"""
    SEMANTIC = "semantic"
    """Query type ``semantic``"""
    VECTOR = "vector"
    """Query type ``vector``"""
    VECTOR_SIMPLE_HYBRID = "vector_simple_hybrid"
    """Query type ``vector_simple_hybrid``"""
    VECTOR_SEMANTIC_HYBRID = "vector_semantic_hybrid"
    """Query type ``vector_semantic_hybrid``"""


class CodeInterpreterOutputType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of CodeInterpreterOutputType."""

    LOGS = "logs"
    IMAGE = "image"


class ComputerActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ComputerActionType."""

    SCREENSHOT = "screenshot"
    CLICK = "click"
    DOUBLE_CLICK = "double_click"
    SCROLL = "scroll"
    TYPE = "type"
    WAIT = "wait"
    KEYPRESS = "keypress"
    DRAG = "drag"
    MOVE = "move"


class ComputerToolCallOutputItemOutputType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """A computer screenshot image used with the computer use tool."""

    SCREENSHOT = "computer_screenshot"


class ConnectionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Type (or category) of the connection."""

    AZURE_OPEN_AI = "AzureOpenAI"
    """Azure OpenAI Service"""
    AZURE_BLOB_STORAGE = "AzureBlob"
    """Azure Blob Storage, with specified container"""
    AZURE_STORAGE_ACCOUNT = "AzureStorageAccount"
    """Azure Blob Storage, with container not specified (used by Agents)"""
    AZURE_AI_SEARCH = "CognitiveSearch"
    """Azure AI Search"""
    COSMOS_DB = "CosmosDB"
    """CosmosDB"""
    API_KEY = "ApiKey"
    """Generic connection that uses API Key authentication"""
    APPLICATION_CONFIGURATION = "AppConfig"
    """Application Configuration"""
    APPLICATION_INSIGHTS = "AppInsights"
    """Application Insights"""
    CUSTOM = "CustomKeys"
    """Custom Keys"""
    REMOTE_TOOL = "RemoteTool"
    """Remote tool"""


class CredentialType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The credential type used by the connection."""

    API_KEY = "ApiKey"
    """API Key credential"""
    ENTRA_ID = "AAD"
    """Entra ID credential (formerly known as AAD)"""
    SAS = "SAS"
    """Shared Access Signature (SAS) credential"""
    CUSTOM = "CustomKeys"
    """Custom credential"""
    NONE = "None"
    """No credential"""
    AGENTIC_IDENTITY = "AgenticIdentityToken"
    """Agentic identity credential"""


class DatasetType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum to determine the type of data."""

    URI_FILE = "uri_file"
    """URI file."""
    URI_FOLDER = "uri_folder"
    """URI folder."""


class DayOfWeek(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Days of the week for recurrence schedule."""

    SUNDAY = "Sunday"
    """Sunday."""
    MONDAY = "Monday"
    """Monday."""
    TUESDAY = "Tuesday"
    """Tuesday."""
    WEDNESDAY = "Wednesday"
    """Wednesday."""
    THURSDAY = "Thursday"
    """Thursday."""
    FRIDAY = "Friday"
    """Friday."""
    SATURDAY = "Saturday"
    """Saturday."""


class DeploymentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of DeploymentType."""

    MODEL_DEPLOYMENT = "ModelDeployment"
    """Model deployment"""


class EvaluationRuleActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the evaluation action."""

    CONTINUOUS_EVALUATION = "continuousEvaluation"
    """Continuous evaluation."""
    HUMAN_EVALUATION = "humanEvaluation"
    """Human evaluation."""


class EvaluationRuleEventType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the evaluation rule event."""

    RESPONSE_COMPLETED = "response.completed"
    """Response completed."""
    MANUAL = "manual"
    """Manual trigger."""


class EvaluationTaxonomyInputType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the evaluation taxonomy input."""

    AGENT = "agent"
    """Agent"""
    POLICY = "policy"
    """Policy."""


class EvaluatorCategory(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The category of the evaluator."""

    QUALITY = "quality"
    """Quality"""
    SAFETY = "safety"
    """Risk & Safety"""
    AGENTS = "agents"
    """Agents"""


class EvaluatorDefinitionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of evaluator definition."""

    PROMPT = "prompt"
    """Prompt-based definition"""
    CODE = "code"
    """Code-based definition"""
    PROMPT_AND_CODE = "prompt_and_code"
    """Prompt & Code Based definition"""
    SERVICE = "service"
    """Service-based evaluator"""
    OPENAI_GRADERS = "openai_graders"
    """OpenAI graders"""


class EvaluatorMetricDirection(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The direction of the metric indicating whether a higher value is better, a lower value is
    better, or neutral.
    """

    INCREASE = "increase"
    """It indicates a higher value is better for this metric"""
    DECREASE = "decrease"
    """It indicates a lower value is better for this metric"""
    NEUTRAL = "neutral"
    """It indicates no preference for this metric direction"""


class EvaluatorMetricType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the evaluator."""

    ORDINAL = "ordinal"
    """Ordinal metric representing categories that can be ordered or ranked."""
    CONTINUOUS = "continuous"
    """Continuous metric representing values in a continuous range."""
    BOOLEAN = "boolean"
    """Boolean metric representing true/false values"""


class EvaluatorType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the evaluator."""

    BUILT_IN = "builtin"
    """Built-in evaluator (Microsoft provided)"""
    CUSTOM = "custom"
    """Custom evaluator"""


class IndexType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of IndexType."""

    AZURE_SEARCH = "AzureSearch"
    """Azure search"""
    COSMOS_DB = "CosmosDBNoSqlVectorStore"
    """CosmosDB"""
    MANAGED_AZURE_SEARCH = "ManagedAzureSearch"
    """Managed Azure Search"""


class InsightType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The request of the insights."""

    EVALUATION_RUN_CLUSTER_INSIGHT = "EvaluationRunClusterInsight"
    """Insights on an Evaluation run result."""
    AGENT_CLUSTER_INSIGHT = "AgentClusterInsight"
    """Cluster Insight on an Agent."""
    EVALUATION_COMPARISON = "EvaluationComparison"
    """Evaluation Comparison."""


class ItemContentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Multi-modal input and output contents."""

    INPUT_TEXT = "input_text"
    INPUT_AUDIO = "input_audio"
    INPUT_IMAGE = "input_image"
    INPUT_FILE = "input_file"
    OUTPUT_TEXT = "output_text"
    OUTPUT_AUDIO = "output_audio"
    REFUSAL = "refusal"


class ItemType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ItemType."""

    MESSAGE = "message"
    FILE_SEARCH_CALL = "file_search_call"
    FUNCTION_CALL = "function_call"
    FUNCTION_CALL_OUTPUT = "function_call_output"
    COMPUTER_CALL = "computer_call"
    COMPUTER_CALL_OUTPUT = "computer_call_output"
    WEB_SEARCH_CALL = "web_search_call"
    REASONING = "reasoning"
    ITEM_REFERENCE = "item_reference"
    IMAGE_GENERATION_CALL = "image_generation_call"
    CODE_INTERPRETER_CALL = "code_interpreter_call"
    LOCAL_SHELL_CALL = "local_shell_call"
    LOCAL_SHELL_CALL_OUTPUT = "local_shell_call_output"
    MCP_LIST_TOOLS = "mcp_list_tools"
    MCP_APPROVAL_REQUEST = "mcp_approval_request"
    MCP_APPROVAL_RESPONSE = "mcp_approval_response"
    MCP_CALL = "mcp_call"
    STRUCTURED_OUTPUTS = "structured_outputs"
    WORKFLOW_ACTION = "workflow_action"
    MEMORY_SEARCH_CALL = "memory_search_call"
    OAUTH_CONSENT_REQUEST = "oauth_consent_request"


class LocationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of LocationType."""

    APPROXIMATE = "approximate"


class MemoryItemKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Memory item kind."""

    USER_PROFILE = "user_profile"
    """User profile information extracted from conversations."""
    CHAT_SUMMARY = "chat_summary"
    """Summary of chat conversations."""


class MemoryOperationKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Memory operation kind."""

    CREATE = "create"
    """Create a new memory item."""
    UPDATE = "update"
    """Update an existing memory item."""
    DELETE = "delete"
    """Delete an existing memory item."""


class MemoryStoreKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of memory store implementation to use."""

    DEFAULT = "default"
    """The default memory store implementation."""


class MemoryStoreUpdateStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of a memory store update operation."""

    QUEUED = "queued"
    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    FAILED = "failed"
    SUPERSEDED = "superseded"


class OpenApiAuthType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Authentication type for OpenApi endpoint. Allowed types are:
    * Anonymous (no authentication required)
    * Project Connection (requires project_connection_id to endpoint, as setup in AI Foundry)
    * Managed_Identity (requires audience for identity based auth).
    """

    ANONYMOUS = "anonymous"
    PROJECT_CONNECTION = "project_connection"
    MANAGED_IDENTITY = "managed_identity"


class OperationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum describing allowed operation states."""

    NOT_STARTED = "NotStarted"
    """The operation has not started."""
    RUNNING = "Running"
    """The operation is in progress."""
    SUCCEEDED = "Succeeded"
    """The operation has completed successfully."""
    FAILED = "Failed"
    """The operation has failed."""
    CANCELED = "Canceled"
    """The operation has been canceled by the user."""


class PendingUploadType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of pending upload."""

    NONE = "None"
    """No pending upload."""
    BLOB_REFERENCE = "BlobReference"
    """Blob Reference is the only supported type."""


class ReasoningEffort(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """**o-series models only**
    Constrains effort on reasoning for
    `reasoning models <https://platform.openai.com/docs/guides/reasoning>`_.
    Currently supported values are ``low``, ``medium``, and ``high``. Reducing
    reasoning effort can result in faster responses and fewer tokens used
    on reasoning in a response.
    """

    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"


class ReasoningItemSummaryPartType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ReasoningItemSummaryPartType."""

    SUMMARY_TEXT = "summary_text"


class RecurrenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Recurrence type."""

    HOURLY = "Hourly"
    """Hourly recurrence pattern."""
    DAILY = "Daily"
    """Daily recurrence pattern."""
    WEEKLY = "Weekly"
    """Weekly recurrence pattern."""
    MONTHLY = "Monthly"
    """Monthly recurrence pattern."""


class ResponseErrorCode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The error code for the response."""

    SERVER_ERROR = "server_error"
    RATE_LIMIT_EXCEEDED = "rate_limit_exceeded"
    INVALID_PROMPT = "invalid_prompt"
    VECTOR_STORE_TIMEOUT = "vector_store_timeout"
    INVALID_IMAGE = "invalid_image"
    INVALID_IMAGE_FORMAT = "invalid_image_format"
    INVALID_BASE64_IMAGE = "invalid_base64_image"
    INVALID_IMAGE_URL = "invalid_image_url"
    IMAGE_TOO_LARGE = "image_too_large"
    IMAGE_TOO_SMALL = "image_too_small"
    IMAGE_PARSE_ERROR = "image_parse_error"
    IMAGE_CONTENT_POLICY_VIOLATION = "image_content_policy_violation"
    INVALID_IMAGE_MODE = "invalid_image_mode"
    IMAGE_FILE_TOO_LARGE = "image_file_too_large"
    UNSUPPORTED_IMAGE_MEDIA_TYPE = "unsupported_image_media_type"
    EMPTY_IMAGE_FILE = "empty_image_file"
    FAILED_TO_DOWNLOAD_IMAGE = "failed_to_download_image"
    IMAGE_FILE_NOT_FOUND = "image_file_not_found"


class ResponsesMessageRole(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The collection of valid roles for responses message items."""

    SYSTEM = "system"
    DEVELOPER = "developer"
    USER = "user"
    ASSISTANT = "assistant"


class ResponseStreamEventType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ResponseStreamEventType."""

    RESPONSE_AUDIO_DELTA = "response.audio.delta"
    RESPONSE_AUDIO_DONE = "response.audio.done"
    RESPONSE_AUDIO_TRANSCRIPT_DELTA = "response.audio_transcript.delta"
    RESPONSE_AUDIO_TRANSCRIPT_DONE = "response.audio_transcript.done"
    RESPONSE_CODE_INTERPRETER_CALL_CODE_DELTA = "response.code_interpreter_call_code.delta"
    RESPONSE_CODE_INTERPRETER_CALL_CODE_DONE = "response.code_interpreter_call_code.done"
    RESPONSE_CODE_INTERPRETER_CALL_COMPLETED = "response.code_interpreter_call.completed"
    RESPONSE_CODE_INTERPRETER_CALL_IN_PROGRESS = "response.code_interpreter_call.in_progress"
    RESPONSE_CODE_INTERPRETER_CALL_INTERPRETING = "response.code_interpreter_call.interpreting"
    RESPONSE_COMPLETED = "response.completed"
    RESPONSE_CONTENT_PART_ADDED = "response.content_part.added"
    RESPONSE_CONTENT_PART_DONE = "response.content_part.done"
    RESPONSE_CREATED = "response.created"
    ERROR = "error"
    RESPONSE_FILE_SEARCH_CALL_COMPLETED = "response.file_search_call.completed"
    RESPONSE_FILE_SEARCH_CALL_IN_PROGRESS = "response.file_search_call.in_progress"
    RESPONSE_FILE_SEARCH_CALL_SEARCHING = "response.file_search_call.searching"
    RESPONSE_FUNCTION_CALL_ARGUMENTS_DELTA = "response.function_call_arguments.delta"
    RESPONSE_FUNCTION_CALL_ARGUMENTS_DONE = "response.function_call_arguments.done"
    RESPONSE_IN_PROGRESS = "response.in_progress"
    RESPONSE_FAILED = "response.failed"
    RESPONSE_INCOMPLETE = "response.incomplete"
    RESPONSE_OUTPUT_ITEM_ADDED = "response.output_item.added"
    RESPONSE_OUTPUT_ITEM_DONE = "response.output_item.done"
    RESPONSE_REFUSAL_DELTA = "response.refusal.delta"
    RESPONSE_REFUSAL_DONE = "response.refusal.done"
    RESPONSE_OUTPUT_TEXT_ANNOTATION_ADDED = "response.output_text.annotation.added"
    RESPONSE_OUTPUT_TEXT_DELTA = "response.output_text.delta"
    RESPONSE_OUTPUT_TEXT_DONE = "response.output_text.done"
    RESPONSE_REASONING_SUMMARY_PART_ADDED = "response.reasoning_summary_part.added"
    RESPONSE_REASONING_SUMMARY_PART_DONE = "response.reasoning_summary_part.done"
    RESPONSE_REASONING_SUMMARY_TEXT_DELTA = "response.reasoning_summary_text.delta"
    RESPONSE_REASONING_SUMMARY_TEXT_DONE = "response.reasoning_summary_text.done"
    RESPONSE_WEB_SEARCH_CALL_COMPLETED = "response.web_search_call.completed"
    RESPONSE_WEB_SEARCH_CALL_IN_PROGRESS = "response.web_search_call.in_progress"
    RESPONSE_WEB_SEARCH_CALL_SEARCHING = "response.web_search_call.searching"
    RESPONSE_IMAGE_GENERATION_CALL_COMPLETED = "response.image_generation_call.completed"
    RESPONSE_IMAGE_GENERATION_CALL_GENERATING = "response.image_generation_call.generating"
    RESPONSE_IMAGE_GENERATION_CALL_IN_PROGRESS = "response.image_generation_call.in_progress"
    RESPONSE_IMAGE_GENERATION_CALL_PARTIAL_IMAGE = "response.image_generation_call.partial_image"
    RESPONSE_MCP_CALL_ARGUMENTS_DELTA = "response.mcp_call.arguments_delta"
    RESPONSE_MCP_CALL_ARGUMENTS_DONE = "response.mcp_call.arguments_done"
    RESPONSE_MCP_CALL_COMPLETED = "response.mcp_call.completed"
    RESPONSE_MCP_CALL_FAILED = "response.mcp_call.failed"
    RESPONSE_MCP_CALL_IN_PROGRESS = "response.mcp_call.in_progress"
    RESPONSE_MCP_LIST_TOOLS_COMPLETED = "response.mcp_list_tools.completed"
    RESPONSE_MCP_LIST_TOOLS_FAILED = "response.mcp_list_tools.failed"
    RESPONSE_MCP_LIST_TOOLS_IN_PROGRESS = "response.mcp_list_tools.in_progress"
    RESPONSE_QUEUED = "response.queued"
    RESPONSE_REASONING_DELTA = "response.reasoning.delta"
    RESPONSE_REASONING_DONE = "response.reasoning.done"
    RESPONSE_REASONING_SUMMARY_DELTA = "response.reasoning_summary.delta"
    RESPONSE_REASONING_SUMMARY_DONE = "response.reasoning_summary.done"


class ResponseTextFormatConfigurationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """An object specifying the format that the model must output.
    Configuring ``{ "type": "json_schema" }`` enables Structured Outputs,
    which ensures the model will match your supplied JSON schema. Learn more in the
    `Structured Outputs guide </docs/guides/structured-outputs>`_.
    The default format is ``{ "type": "text" }`` with no additional options.
    **Not recommended for gpt-4o and newer models:**
    Setting to ``{ "type": "json_object" }`` enables the older JSON mode, which
    ensures the message the model generates is valid JSON. Using ``json_schema``
    is preferred for models that support it.
    """

    TEXT = "text"
    JSON_SCHEMA = "json_schema"
    JSON_OBJECT = "json_object"


class RiskCategory(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Risk category for the attack objective."""

    HATE_UNFAIRNESS = "HateUnfairness"
    """Represents content related to hate or unfairness."""
    VIOLENCE = "Violence"
    """Represents content related to violence."""
    SEXUAL = "Sexual"
    """Represents content of a sexual nature."""
    SELF_HARM = "SelfHarm"
    """Represents content related to self-harm."""


class SampleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of sample used in the analysis."""

    EVALUATION_RESULT_SAMPLE = "EvaluationResultSample"
    """A sample from the evaluation result."""


class ScheduleProvisioningStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Schedule provisioning status."""

    CREATING = "Creating"
    """Represents the creation status of the schedule."""
    UPDATING = "Updating"
    """Represents the updating status of the schedule."""
    DELETING = "Deleting"
    """Represents the deleting status of the schedule."""
    SUCCEEDED = "Succeeded"
    """Represents the succeeded status of the schedule."""
    FAILED = "Failed"
    """Represents the failed status of the schedule."""


class ScheduleTaskType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the task."""

    EVALUATION = "Evaluation"
    """Evaluation task."""
    INSIGHT = "Insight"
    """Insight task."""


class ServiceTier(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the processing type used for serving the request.
    * If set to 'auto', then the request will be processed with the service tier configured in the
      Project settings. Unless otherwise configured, the Project will use 'default'.
    * If set to 'default', then the request will be processed with the standard pricing and
      performance for the selected model.
    * If set to '[flex](/docs/guides/flex-processing)' or 'priority', then the request will be
      processed with the corresponding service tier. [Contact
      sales](https://openai.com/contact-sales) to learn more about Priority processing.
    * When not set, the default behavior is 'auto'.
      When the ``service_tier`` parameter is set, the response body will include the
        ``service_tier`` value based on the processing mode actually used to serve the request. This
        response value may be different from the value set in the parameter.
    """

    AUTO = "auto"
    DEFAULT = "default"
    FLEX = "flex"
    SCALE = "scale"
    PRIORITY = "priority"


class ToolChoiceObjectType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indicates that the model should use a built-in tool to generate a response.
    `Learn more about built-in tools </docs/guides/tools>`_.
    """

    FILE_SEARCH = "file_search"
    FUNCTION = "function"
    COMPUTER = "computer_use_preview"
    WEB_SEARCH = "web_search_preview"
    IMAGE_GENERATION = "image_generation"
    CODE_INTERPRETER = "code_interpreter"
    MCP = "mcp"


class ToolChoiceOptions(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Controls which (if any) tool is called by the model.
    ``none`` means the model will not call any tool and instead generates a message.
    ``auto`` means the model can pick between generating a message or calling one or
    more tools.
    ``required`` means the model must call one or more tools.
    """

    NONE = "none"
    AUTO = "auto"
    REQUIRED = "required"


class ToolType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """A tool that can be used to generate a response."""

    FILE_SEARCH = "file_search"
    FUNCTION = "function"
    COMPUTER_USE_PREVIEW = "computer_use_preview"
    WEB_SEARCH_PREVIEW = "web_search_preview"
    MCP = "mcp"
    CODE_INTERPRETER = "code_interpreter"
    IMAGE_GENERATION = "image_generation"
    LOCAL_SHELL = "local_shell"
    BING_GROUNDING = "bing_grounding"
    BROWSER_AUTOMATION_PREVIEW = "browser_automation_preview"
    FABRIC_DATAAGENT_PREVIEW = "fabric_dataagent_preview"
    SHAREPOINT_GROUNDING_PREVIEW = "sharepoint_grounding_preview"
    AZURE_AI_SEARCH = "azure_ai_search"
    OPENAPI = "openapi"
    BING_CUSTOM_SEARCH_PREVIEW = "bing_custom_search_preview"
    CAPTURE_STRUCTURED_OUTPUTS = "capture_structured_outputs"
    A2_A_PREVIEW = "a2a_preview"
    AZURE_FUNCTION = "azure_function"
    MEMORY_SEARCH = "memory_search"


class TreatmentEffectType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Treatment Effect Type."""

    TOO_FEW_SAMPLES = "TooFewSamples"
    """Not enough samples to determine treatment effect."""
    INCONCLUSIVE = "Inconclusive"
    """No significant difference between treatment and baseline."""
    CHANGED = "Changed"
    """Indicates the metric changed with statistical significance, but the direction is neutral."""
    IMPROVED = "Improved"
    """Indicates the treatment significantly improved the metric compared to baseline."""
    DEGRADED = "Degraded"
    """Indicates the treatment significantly degraded the metric compared to baseline."""


class TriggerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the trigger."""

    CRON = "Cron"
    """Cron based trigger."""
    RECURRENCE = "Recurrence"
    """Recurrence based trigger."""
    ONE_TIME = "OneTime"
    """One-time trigger."""


class WebSearchActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of WebSearchActionType."""

    SEARCH = "search"
    OPEN_PAGE = "open_page"
    FIND = "find"



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from dataclasses import dataclass, field

from azure.core.credentials_async import AsyncTokenCredential


@dataclass(frozen=True)
class HttpServerConfiguration:
    """Resolved configuration for the HTTP server.

    :ivar str host: The host address the server listens on. Defaults to '0.0.0.0'.
    :ivar int port: The port number the server listens on. Defaults to 8088.
    """

    host: str = "0.0.0.0"
    port: int = 8088


class ToolsConfiguration:
    """Resolved configuration for the Tools subsystem.

    :ivar int catalog_cache_ttl: The time-to-live (TTL) for the tool catalog cache in seconds.
        Defaults to 600 seconds (10 minutes).
    :ivar int catalog_cache_max_size: The maximum size of the tool catalog cache.
        Defaults to 1024 entries.
    """

    catalog_cache_ttl: int = 600
    catalog_cache_max_size: int = 1024


@dataclass(frozen=True)
class AgentServerConfiguration:
    """Resolved configuration for the Agent Server application."""

    project_endpoint: str
    credential: AsyncTokenCredential
    agent_name: str = "$default"
    http: HttpServerConfiguration = field(default_factory=HttpServerConfiguration)
    tools: ToolsConfiguration = field(default_factory=ToolsConfiguration)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
import re
from typing import Any, Dict, Union

from .. import FoundryConnectedTool, FoundryHostedMcpTool
from .._exceptions import InvalidToolFacadeError
from ..client._models import FoundryTool, FoundryToolProtocol

# FoundryToolFacade: a “tool descriptor” bag.
#
# Reserved keys:
#   Required:
#     - "type": str            Discriminator, e.g. "mcp" | "a2a" | "code_interpreter" | ...
#   Optional:
#     - "project_connection_id": str     Project connection id of Foundry connected tools,
#                                        required when "type" is "mcp" or "a2a".
#
# Custom keys:
#   - Allowed, but MUST NOT shadow reserved keys.
FoundryToolFacade = Dict[str, Any]

FoundryToolLike = Union[FoundryToolFacade, FoundryTool]


def ensure_foundry_tool(tool: FoundryToolLike) -> FoundryTool:
    """Ensure the input is a FoundryTool instance.

    :param tool: The tool descriptor, either as a FoundryToolFacade or FoundryTool.
    :type tool: FoundryToolLike
    :return: The corresponding FoundryTool instance.
    :rtype: FoundryTool
    """
    if isinstance(tool, FoundryTool):
        return tool

    tool = tool.copy()
    tool_type = tool.pop("type", None)
    if not isinstance(tool_type, str) or not tool_type:
        raise InvalidToolFacadeError("FoundryToolFacade must have a valid 'type' field of type str.")

    try:
        protocol = FoundryToolProtocol(tool_type)
        project_connection_id = tool.pop("project_connection_id", None)
        if not isinstance(project_connection_id, str) or not project_connection_id:
            raise InvalidToolFacadeError(f"project_connection_id is required for tool protocol {protocol}.")

        # Parse the connection identifier to extract the connection name
        connection_name = _parse_connection_id(project_connection_id)
        return FoundryConnectedTool(protocol=protocol, project_connection_id=connection_name)
    except ValueError:
        return FoundryHostedMcpTool(name=tool_type, configuration=tool)


# Pattern for Azure resource ID format:
# /subscriptions/<sub>/resourceGroups/<rg>/providers/Microsoft.CognitiveServices/accounts
#    /<account>/projects/<project>/connections/<name>
_RESOURCE_ID_PATTERN = re.compile(
    r"^/subscriptions/[^/]+/resourceGroups/[^/]+/providers/Microsoft\.CognitiveServices/"
    r"accounts/[^/]+/projects/[^/]+/connections/(?P<name>[^/]+)$",
    re.IGNORECASE,
)


def _parse_connection_id(connection_id: str) -> str:
    """Parse the connection identifier and extract the connection name.

    Supports two formats:
    1. Simple name: "my-connection-name"
    2. Resource ID: "/subscriptions/<sub>/resourceGroups/<rg>/providers
            /Microsoft.CognitiveServices/accounts/<account>/projects/<project>/connections/<name>"

    :param connection_id: The connection identifier, either a simple name or a full resource ID.
    :type connection_id: str
    :return: The connection name extracted from the identifier.
    :rtype: str
    :raises InvalidToolFacadeError: If the connection_id format is invalid.
    """
    if not connection_id:
        raise InvalidToolFacadeError("Connection identifier cannot be empty.")

    # Check if it's a resource ID format (starts with /)
    if connection_id.startswith("/"):
        match = _RESOURCE_ID_PATTERN.match(connection_id)
        if not match:
            raise InvalidToolFacadeError(
                f"Invalid resource ID format for connection: '{connection_id}'. "
                "Expected format: /subscriptions/<sub>/resourceGroups/<rg>/providers/"
                "Microsoft.CognitiveServices/accounts/<account>/projects/<project>/connections/<name>"
            )
        return match.group("name")

    # Otherwise, treat it as a simple connection name
    return connection_id



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

__path__ = __import__('pkgutil').extend_path(__path__, __name__)

from .client._client import FoundryToolClient
from ._exceptions import (
    ToolInvocationError,
    OAuthConsentRequiredError,
    UnableToResolveToolInvocationError,
    InvalidToolFacadeError,
)
from .client._models import (
    FoundryConnectedTool,
    FoundryHostedMcpTool,
    FoundryTool,
    FoundryToolDetails,
    FoundryToolProtocol,
    FoundryToolSource,
    ResolvedFoundryTool,
    SchemaDefinition,
    SchemaProperty,
    SchemaType,
    UserInfo,
)
from .runtime._catalog import (
    FoundryToolCatalog,
    CachedFoundryToolCatalog,
    DefaultFoundryToolCatalog,
)
from .runtime._facade import FoundryToolFacade, FoundryToolLike, ensure_foundry_tool
from .runtime._invoker import FoundryToolInvoker, DefaultFoundryToolInvoker
from .runtime._resolver import FoundryToolInvocationResolver, DefaultFoundryToolInvocationResolver
from .runtime._runtime import FoundryToolRuntime, DefaultFoundryToolRuntime
from .runtime._starlette import UserInfoContextMiddleware
from .runtime._user import UserProvider, ContextVarUserProvider

__all__ = [
    # Client
    "FoundryToolClient",
    # Exceptions
    "ToolInvocationError",
    "OAuthConsentRequiredError",
    "UnableToResolveToolInvocationError",
    "InvalidToolFacadeError",
    # Models
    "FoundryConnectedTool",
    "FoundryHostedMcpTool",
    "FoundryTool",
    "FoundryToolDetails",
    "FoundryToolProtocol",
    "FoundryToolSource",
    "ResolvedFoundryTool",
    "SchemaDefinition",
    "SchemaProperty",
    "SchemaType",
    "UserInfo",
    # Catalog
    "FoundryToolCatalog",
    "CachedFoundryToolCatalog",
    "DefaultFoundryToolCatalog",
    # Facade
    "FoundryToolFacade",
    "FoundryToolLike",
    "ensure_foundry_tool",
    # Invoker
    "FoundryToolInvoker",
    "DefaultFoundryToolInvoker",
    # Resolver
    "FoundryToolInvocationResolver",
    "DefaultFoundryToolInvocationResolver",
    # Runtime
    "FoundryToolRuntime",
    "DefaultFoundryToolRuntime",
    # Starlette
    "UserInfoContextMiddleware",
    # User
    "UserProvider",
    "ContextVarUserProvider",
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for _catalog.py - testing public methods of DefaultFoundryToolCatalog."""
import asyncio
import pytest
from unittest.mock import AsyncMock

from azure.ai.agentserver.core.tools import ensure_foundry_tool
from azure.ai.agentserver.core.tools.runtime._catalog import (
    DefaultFoundryToolCatalog,
)
from azure.ai.agentserver.core.tools.client._models import (
    FoundryToolDetails,
    ResolvedFoundryTool,
    UserInfo,
)


class TestFoundryToolCatalogGet:
    """Tests for FoundryToolCatalog.get method."""

    @pytest.mark.asyncio
    async def test_get_returns_resolved_tool_when_found(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool,
        sample_tool_details,
        sample_user_info
    ):
        """Test get returns a resolved tool when the tool is found."""
        mock_foundry_tool_client.list_tools_details = AsyncMock(
            return_value={sample_hosted_mcp_tool.id: [sample_tool_details]}
        )

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        result = await catalog.get(sample_hosted_mcp_tool)

        assert result is not None
        assert isinstance(result, ResolvedFoundryTool)
        assert result.details == sample_tool_details

    @pytest.mark.asyncio
    async def test_get_returns_none_when_not_found(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool
    ):
        """Test get returns None when the tool is not found."""
        mock_foundry_tool_client.list_tools_details = AsyncMock(return_value={})

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        result = await catalog.get(sample_hosted_mcp_tool)

        assert result is None


class TestDefaultFoundryToolCatalogList:
    """Tests for DefaultFoundryToolCatalog.list method."""

    @pytest.mark.asyncio
    async def test_list_returns_empty_list_when_no_tools(
        self,
        mock_foundry_tool_client,
        mock_user_provider
    ):
        """Test list returns empty list when no tools are provided."""
        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        result = await catalog.list([])

        assert result == []

    @pytest.mark.asyncio
    async def test_list_returns_resolved_tools(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool,
        sample_tool_details
    ):
        """Test list returns resolved tools."""
        mock_foundry_tool_client.list_tools_details = AsyncMock(
            return_value={sample_hosted_mcp_tool.id: [sample_tool_details]}
        )

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        result = await catalog.list([sample_hosted_mcp_tool])

        assert len(result) == 1
        assert isinstance(result[0], ResolvedFoundryTool)
        assert result[0].definition == sample_hosted_mcp_tool
        assert result[0].details == sample_tool_details

    @pytest.mark.asyncio
    async def test_list_multiple_tools_with_multiple_details(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool,
        sample_connected_tool,
        sample_schema_definition
    ):
        """Test list returns all resolved tools when tools have multiple details."""
        details1 = FoundryToolDetails(
            name="tool1",
            description="First tool",
            input_schema=sample_schema_definition
        )
        details2 = FoundryToolDetails(
            name="tool2",
            description="Second tool",
            input_schema=sample_schema_definition
        )

        mock_foundry_tool_client.list_tools_details = AsyncMock(
            return_value={
                sample_hosted_mcp_tool.id: [details1],
                sample_connected_tool.id: [details2]
            }
        )

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        result = await catalog.list([sample_hosted_mcp_tool, sample_connected_tool])

        assert len(result) == 2
        names = {r.details.name for r in result}
        assert names == {"tool1", "tool2"}

    @pytest.mark.asyncio
    async def test_list_caches_results_for_hosted_mcp_tools(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool,
        sample_tool_details
    ):
        """Test that list caches results for hosted MCP tools."""
        mock_foundry_tool_client.list_tools_details = AsyncMock(
            return_value={sample_hosted_mcp_tool.id: [sample_tool_details]}
        )

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        # First call
        result1 = await catalog.list([sample_hosted_mcp_tool])
        # Second call should use cache
        result2 = await catalog.list([sample_hosted_mcp_tool])

        # Client should only be called once
        assert mock_foundry_tool_client.list_tools_details.call_count == 1
        assert len(result1) == len(result2) == 1

    @pytest.mark.asyncio
    async def test_list_with_facade_dict(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_tool_details
    ):
        """Test list works with facade dictionaries."""
        facade = {"type": "custom_tool", "config": "value"}
        expected_id = ensure_foundry_tool(facade).id

        mock_foundry_tool_client.list_tools_details = AsyncMock(
            return_value={expected_id: [sample_tool_details]}
        )

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        result = await catalog.list([facade])

        assert len(result) == 1
        assert result[0].details == sample_tool_details

    @pytest.mark.asyncio
    async def test_list_returns_multiple_details_per_tool(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool,
        sample_schema_definition
    ):
        """Test list returns multiple resolved tools when a tool has multiple details."""
        details1 = FoundryToolDetails(
            name="function1",
            description="First function",
            input_schema=sample_schema_definition
        )
        details2 = FoundryToolDetails(
            name="function2",
            description="Second function",
            input_schema=sample_schema_definition
        )

        mock_foundry_tool_client.list_tools_details = AsyncMock(
            return_value={sample_hosted_mcp_tool.id: [details1, details2]}
        )

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        result = await catalog.list([sample_hosted_mcp_tool])

        assert len(result) == 2
        names = {r.details.name for r in result}
        assert names == {"function1", "function2"}

    @pytest.mark.asyncio
    async def test_list_handles_exception_from_client(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool
    ):
        """Test list propagates exception from client and clears cache."""
        mock_foundry_tool_client.list_tools_details = AsyncMock(
            side_effect=RuntimeError("Network error")
        )

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        with pytest.raises(RuntimeError, match="Network error"):
            await catalog.list([sample_hosted_mcp_tool])

    @pytest.mark.asyncio
    async def test_list_connected_tool_cache_key_includes_user(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_connected_tool,
        sample_tool_details,
        sample_user_info
    ):
        """Test that connected tool cache key includes user info."""
        mock_foundry_tool_client.list_tools_details = AsyncMock(
            return_value={sample_connected_tool.id: [sample_tool_details]}
        )

        # Create a new user provider returning a different user
        other_user = UserInfo(object_id="other-oid", tenant_id="other-tid")
        mock_user_provider2 = AsyncMock()
        mock_user_provider2.get_user = AsyncMock(return_value=other_user)

        catalog1 = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        catalog2 = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider2,
            agent_name="test-agent"
        )

        # Both catalogs should be able to list tools
        result1 = await catalog1.list([sample_connected_tool])
        result2 = await catalog2.list([sample_connected_tool])

        assert len(result1) == 1
        assert len(result2) == 1


class TestCachedFoundryToolCatalogConcurrency:
    """Tests for CachedFoundryToolCatalog concurrency handling."""

    @pytest.mark.asyncio
    async def test_concurrent_requests_share_single_fetch(
        self,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool,
        sample_tool_details
    ):
        """Test that concurrent requests for the same tool share a single fetch."""
        call_count = 0
        fetch_event = asyncio.Event()

        async def slow_fetch(*args, **kwargs):
            nonlocal call_count
            call_count += 1
            await fetch_event.wait()
            return {sample_hosted_mcp_tool.id: [sample_tool_details]}

        mock_foundry_tool_client.list_tools_details = slow_fetch

        catalog = DefaultFoundryToolCatalog(
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        # Start two concurrent requests
        task1 = asyncio.create_task(catalog.list([sample_hosted_mcp_tool]))
        task2 = asyncio.create_task(catalog.list([sample_hosted_mcp_tool]))

        # Allow tasks to start
        await asyncio.sleep(0.01)

        # Release the fetch
        fetch_event.set()

        results = await asyncio.gather(task1, task2)

        # Both should get results, but fetch should only be called once
        assert len(results[0]) == 1
        assert len(results[1]) == 1
        assert call_count == 1



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from contextvars import ContextVar
from abc import ABC, abstractmethod
from typing import ClassVar, Mapping, Optional

from ..client._models import UserInfo


class UserProvider(ABC):
    """Base class for user providers."""

    @abstractmethod
    async def get_user(self) -> Optional[UserInfo]:
        """Get the user information.

        :return: The user information or None if not found.
        :rtype: Optional[UserInfo]
        """
        raise NotImplementedError


class ContextVarUserProvider(UserProvider):
    """User provider that retrieves user information from a ContextVar."""
    default_user_info_context: ClassVar[ContextVar[UserInfo]] = ContextVar("user_info_context")

    def __init__(self, context: Optional[ContextVar[UserInfo]] = None):
        self.context = context or self.default_user_info_context

    async def get_user(self) -> Optional[UserInfo]:
        """Get the user information from the context variable.

        :return: The user information or None if not found.
        :rtype: Optional[UserInfo]
        """
        return self.context.get(None)


def resolve_user_from_headers(headers: Mapping[str, str],
                              object_id_header: str = "x-aml-oid",
                              tenant_id_header: str = "x-aml-tid") -> Optional[UserInfo]:
    """Resolve user information from HTTP headers.

    :param headers: The HTTP headers.
    :type headers: Mapping[str, str]
    :param object_id_header: The header name for the object ID.
    :type object_id_header: str
    :param tenant_id_header: The header name for the tenant ID.
    :type tenant_id_header: str
    :return: The user information or None if not found.
    :rtype: Optional[UserInfo]
    """
    object_id = headers.get(object_id_header, "")
    tenant_id = headers.get(tenant_id_header, "")

    if not object_id or not tenant_id:
        return None

    return UserInfo(object_id=object_id, tenant_id=tenant_id)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from abc import ABC, abstractmethod
from typing import Any, Dict

from ._user import UserProvider
from ..client._client import FoundryToolClient
from ..client._models import ResolvedFoundryTool


class FoundryToolInvoker(ABC):
    """Abstract base class for Foundry tool invokers."""

    @property
    @abstractmethod
    def resolved_tool(self) -> ResolvedFoundryTool:
        """Get the resolved tool definition.

        :return: The tool definition.
        :rtype: ResolvedFoundryTool
        """
        raise NotImplementedError

    @abstractmethod
    async def invoke(self, arguments: Dict[str, Any]) -> Any:
        """Invoke the tool with the given arguments.

        :param arguments: The arguments to pass to the tool.
        :type arguments: Dict[str, Any]
        :return: The result of the tool invocation
        :rtype: Any
        """
        raise NotImplementedError


class DefaultFoundryToolInvoker(FoundryToolInvoker):
    """Default implementation of FoundryToolInvoker."""

    def __init__(self,
                 resolved_tool: ResolvedFoundryTool,
                 client: FoundryToolClient,
                 user_provider: UserProvider,
                 agent_name: str):
        self._resolved_tool = resolved_tool
        self._client = client
        self._user_provider = user_provider
        self._agent_name = agent_name

    @property
    def resolved_tool(self) -> ResolvedFoundryTool:
        """Get the resolved tool definition.

        :return: The tool definition.
        :rtype: ResolvedFoundryTool
        """
        return self._resolved_tool

    async def invoke(self, arguments: Dict[str, Any]) -> Any:
        """Invoke the tool with the given arguments.

        :param arguments: The arguments to pass to the tool
        :type arguments: Dict[str, Any]
        :return: The result of the tool invocation
        :rtype: Any
        """
        user = await self._user_provider.get_user()
        result = await self._client.invoke_tool(self._resolved_tool, arguments, self._agent_name, user)
        return result



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from abc import ABC, abstractmethod
from typing import Union

from ._catalog import FoundryToolCatalog
from ._facade import FoundryToolLike, ensure_foundry_tool
from ._invoker import DefaultFoundryToolInvoker, FoundryToolInvoker
from ._user import UserProvider
from .. import FoundryToolClient
from .._exceptions import UnableToResolveToolInvocationError
from ..client._models import ResolvedFoundryTool


class FoundryToolInvocationResolver(ABC):
    """Resolver for Foundry tool invocations."""

    @abstractmethod
    async def resolve(self, tool: Union[FoundryToolLike, ResolvedFoundryTool]) -> FoundryToolInvoker:
        """Resolves a Foundry tool invocation.

        :param tool: The Foundry tool to resolve.
        :type tool: Union[FoundryToolLike, ResolvedFoundryTool]
        :return: The resolved Foundry tool invoker.
        :rtype: FoundryToolInvoker
        """
        raise NotImplementedError


class DefaultFoundryToolInvocationResolver(FoundryToolInvocationResolver):
    """Default implementation of FoundryToolInvocationResolver."""

    def __init__(self,
                 catalog: FoundryToolCatalog,
                 client: FoundryToolClient,
                 user_provider: UserProvider,
                 agent_name: str):
        self._catalog = catalog
        self._client = client
        self._user_provider = user_provider
        self._agent_name = agent_name

    async def resolve(self, tool: Union[FoundryToolLike, ResolvedFoundryTool]) -> FoundryToolInvoker:
        """Resolves a Foundry tool invocation.

        :param tool: The Foundry tool to resolve.
        :type tool: Union[FoundryToolLike, ResolvedFoundryTool]
        :return: The resolved Foundry tool invoker.
        :rtype: FoundryToolInvoker
        """
        if isinstance(tool, ResolvedFoundryTool):
            resolved_tool = tool
        else:
            foundry_tool = ensure_foundry_tool(tool)
            resolved_tool = await self._catalog.get(foundry_tool)  # type: ignore[assignment]
            if not resolved_tool:
                raise UnableToResolveToolInvocationError(f"Unable to resolve tool {foundry_tool} from catalog",
                                                         foundry_tool)
        return DefaultFoundryToolInvoker(resolved_tool, self._client, self._user_provider, self._agent_name)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for _runtime.py - testing public methods of DefaultFoundryToolRuntime."""
import os
import pytest
from unittest.mock import AsyncMock, MagicMock, patch

from azure.ai.agentserver.core.tools.runtime._runtime import DefaultFoundryToolRuntime
from azure.ai.agentserver.core.tools.runtime._catalog import DefaultFoundryToolCatalog
from azure.ai.agentserver.core.tools.runtime._resolver import DefaultFoundryToolInvocationResolver
from azure.ai.agentserver.core.tools.runtime._user import ContextVarUserProvider


class TestDefaultFoundryToolRuntimeInit:
    """Tests for DefaultFoundryToolRuntime initialization."""

    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    def test_init_creates_client_with_endpoint_and_credential(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test initialization creates client with correct endpoint and credential."""
        endpoint = "https://test-project.azure.com"
        mock_client_class.return_value = MagicMock()

        runtime = DefaultFoundryToolRuntime(
            project_endpoint=endpoint,
            credential=mock_credential
        )

        mock_client_class.assert_called_once_with(
            endpoint=endpoint,
            credential=mock_credential
        )
        assert runtime is not None

    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    def test_init_uses_default_user_provider_when_none_provided(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test initialization uses ContextVarUserProvider when user_provider is None."""
        mock_client_class.return_value = MagicMock()

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        assert isinstance(runtime._user_provider, ContextVarUserProvider)

    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    def test_init_uses_custom_user_provider(
        self,
        mock_client_class,
        mock_credential,
        mock_user_provider
    ):
        """Test initialization uses custom user provider when provided."""
        mock_client_class.return_value = MagicMock()

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential,
            user_provider=mock_user_provider
        )

        assert runtime._user_provider is mock_user_provider

    @patch.dict(os.environ, {"AGENT_NAME": "custom-agent"})
    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    def test_init_reads_agent_name_from_environment(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test initialization reads agent name from environment variable."""
        mock_client_class.return_value = MagicMock()

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        assert runtime._agent_name == "custom-agent"

    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    def test_init_uses_default_agent_name_when_env_not_set(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test initialization uses default agent name when env var is not set."""
        mock_client_class.return_value = MagicMock()

        # Ensure AGENT_NAME is not set
        env_copy = os.environ.copy()
        if "AGENT_NAME" in env_copy:
            del env_copy["AGENT_NAME"]

        with patch.dict(os.environ, env_copy, clear=True):
            runtime = DefaultFoundryToolRuntime(
                project_endpoint="https://test.azure.com",
                credential=mock_credential
            )

            assert runtime._agent_name == "$default"


class TestDefaultFoundryToolRuntimeCatalog:
    """Tests for DefaultFoundryToolRuntime.catalog property."""

    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    def test_catalog_returns_default_catalog(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test catalog property returns DefaultFoundryToolCatalog."""
        mock_client_class.return_value = MagicMock()

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        assert isinstance(runtime.catalog, DefaultFoundryToolCatalog)


class TestDefaultFoundryToolRuntimeInvocation:
    """Tests for DefaultFoundryToolRuntime.invocation property."""

    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    def test_invocation_returns_default_resolver(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test invocation property returns DefaultFoundryToolInvocationResolver."""
        mock_client_class.return_value = MagicMock()

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        assert isinstance(runtime.invocation, DefaultFoundryToolInvocationResolver)


class TestDefaultFoundryToolRuntimeInvoke:
    """Tests for DefaultFoundryToolRuntime.invoke method."""

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    async def test_invoke_resolves_and_invokes_tool(
        self,
        mock_client_class,
        mock_credential,
        sample_resolved_mcp_tool
    ):
        """Test invoke resolves the tool and calls the invoker."""
        mock_client_instance = MagicMock()
        mock_client_class.return_value = mock_client_instance

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        # Mock the invocation resolver
        mock_invoker = AsyncMock()
        mock_invoker.invoke = AsyncMock(return_value={"result": "success"})
        runtime._invocation.resolve = AsyncMock(return_value=mock_invoker)

        result = await runtime.invoke(sample_resolved_mcp_tool, {"input": "test"})

        assert result == {"result": "success"}
        runtime._invocation.resolve.assert_called_once_with(sample_resolved_mcp_tool)
        mock_invoker.invoke.assert_called_once_with({"input": "test"})

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    async def test_invoke_with_facade_dict(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test invoke works with facade dictionary."""
        mock_client_instance = MagicMock()
        mock_client_class.return_value = mock_client_instance

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        facade = {"type": "custom_tool", "config": "value"}

        # Mock the invocation resolver
        mock_invoker = AsyncMock()
        mock_invoker.invoke = AsyncMock(return_value={"output": "done"})
        runtime._invocation.resolve = AsyncMock(return_value=mock_invoker)

        result = await runtime.invoke(facade, {"param": "value"})

        assert result == {"output": "done"}
        runtime._invocation.resolve.assert_called_once_with(facade)


class TestDefaultFoundryToolRuntimeContextManager:
    """Tests for DefaultFoundryToolRuntime async context manager."""

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    async def test_aenter_returns_runtime_and_enters_client(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test __aenter__ enters client and returns runtime."""
        mock_client_instance = AsyncMock()
        mock_client_instance.__aenter__ = AsyncMock(return_value=mock_client_instance)
        mock_client_instance.__aexit__ = AsyncMock(return_value=None)
        mock_client_class.return_value = mock_client_instance

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        async with runtime as r:
            assert r is runtime
            mock_client_instance.__aenter__.assert_called_once()

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    async def test_aexit_exits_client(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test __aexit__ exits client properly."""
        mock_client_instance = AsyncMock()
        mock_client_instance.__aenter__ = AsyncMock(return_value=mock_client_instance)
        mock_client_instance.__aexit__ = AsyncMock(return_value=None)
        mock_client_class.return_value = mock_client_instance

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        async with runtime:
            pass

        mock_client_instance.__aexit__.assert_called_once()

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.runtime._runtime.FoundryToolClient")
    async def test_aexit_called_on_exception(
        self,
        mock_client_class,
        mock_credential
    ):
        """Test __aexit__ is called even when exception occurs."""
        mock_client_instance = AsyncMock()
        mock_client_instance.__aenter__ = AsyncMock(return_value=mock_client_instance)
        mock_client_instance.__aexit__ = AsyncMock(return_value=None)
        mock_client_class.return_value = mock_client_instance

        runtime = DefaultFoundryToolRuntime(
            project_endpoint="https://test.azure.com",
            credential=mock_credential
        )

        with pytest.raises(ValueError):
            async with runtime:
                raise ValueError("Test error")

        mock_client_instance.__aexit__.assert_called_once()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
import os
from typing import Any, AsyncContextManager, Dict, Optional, Union

from azure.core.credentials_async import AsyncTokenCredential

from ._catalog import DefaultFoundryToolCatalog, FoundryToolCatalog
from ._facade import FoundryToolLike
from ._resolver import DefaultFoundryToolInvocationResolver, FoundryToolInvocationResolver
from ._user import ContextVarUserProvider, UserProvider
from ..client._models import ResolvedFoundryTool
from ..client._client import FoundryToolClient
from ...constants import Constants


class FoundryToolRuntime(AsyncContextManager["FoundryToolRuntime"]):
    """Base class for Foundry tool runtimes."""

    @property
    def catalog(self) -> FoundryToolCatalog:
        """The tool catalog.

        :return: The tool catalog.
        :rtype: FoundryToolCatalog
        """
        raise NotImplementedError

    @property
    def invocation(self) -> FoundryToolInvocationResolver:
        """The tool invocation resolver.

        :return: The tool invocation resolver.
        :rtype: FoundryToolInvocationResolver
        """
        raise NotImplementedError

    async def invoke(self, tool: Union[FoundryToolLike, ResolvedFoundryTool], arguments: Dict[str, Any]) -> Any:
        """Invoke a tool with the given arguments.

        :param tool: The tool to invoke.
        :type tool: Union[FoundryToolLike, ResolvedFoundryTool]
        :param arguments: The arguments to pass to the tool.
        :type arguments: Dict[str, Any]
        :return: The result of the tool invocation.
        :rtype: Any
        """
        invoker = await self.invocation.resolve(tool)
        return await invoker.invoke(arguments)


class DefaultFoundryToolRuntime(FoundryToolRuntime):
    """Default implementation of FoundryToolRuntime."""

    def __init__(self,
                 project_endpoint: str,
                 credential: "AsyncTokenCredential",
                 user_provider: Optional[UserProvider] = None):
        # Do we need introduce DI here?
        self._user_provider = user_provider or ContextVarUserProvider()
        self._agent_name = os.getenv(Constants.AGENT_NAME, "$default")
        self._client = FoundryToolClient(endpoint=project_endpoint, credential=credential)
        self._catalog = DefaultFoundryToolCatalog(client=self._client,
                                                  user_provider=self._user_provider,
                                                  agent_name=self._agent_name)
        self._invocation = DefaultFoundryToolInvocationResolver(catalog=self._catalog,
                                                                client=self._client,
                                                                user_provider=self._user_provider,
                                                                agent_name=self._agent_name)

    @property
    def catalog(self) -> FoundryToolCatalog:
        """The tool catalog.

        :rtype: FoundryToolCatalog
        """
        return self._catalog

    @property
    def invocation(self) -> FoundryToolInvocationResolver:
        """The tool invocation resolver.

        :rtype: FoundryToolInvocationResolver
        """
        return self._invocation

    async def __aenter__(self) -> "DefaultFoundryToolRuntime":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        await self._client.__aexit__(exc_type, exc_value, traceback)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from abc import ABC
from typing import Any, AsyncIterable, ClassVar, Dict, Iterable, List, Mapping, Optional, Tuple, cast

from azure.core.pipeline.transport import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async

from ._base import BaseOperations
from .._models import FoundryConnectedTool, FoundryToolDetails, FoundryToolSource, InvokeFoundryConnectedToolsResponse, \
	ListFoundryConnectedToolsResponse, ResolvedFoundryTool, UserInfo
from ..._exceptions import ToolInvocationError


class BaseFoundryConnectedToolsOperations(BaseOperations, ABC):
	"""Base operations for Foundry connected tools."""

	_API_VERSION: ClassVar[str] = "2025-11-15-preview"

	_HEADERS: ClassVar[Dict[str, str]] = {
		"Content-Type": "application/json",
		"Accept": "application/json",
	}

	_QUERY_PARAMS: ClassVar[Dict[str, Any]] = {
		"api-version": _API_VERSION
	}

	@staticmethod
	def _list_tools_path(agent_name: str) -> str:
		return f"/agents/{agent_name}/tools/resolve"

	@staticmethod
	def _invoke_tool_path(agent_name: str) -> str:
		return f"/agents/{agent_name}/tools/invoke"

	def _build_list_tools_request(
			self,
			tools: List[FoundryConnectedTool],
			user: Optional[UserInfo],
			agent_name: str,) -> HttpRequest:
		payload: Dict[str, Any] = {
			"remoteServers": [
				{
					"projectConnectionId": tool.project_connection_id,
					"protocol": tool.protocol,
				} for tool in tools
			],
		}
		if user:
			payload["user"] = {
				"objectId": user.object_id,
				"tenantId": user.tenant_id,
			}
		return self._client.post(
			self._list_tools_path(agent_name),
			params=self._QUERY_PARAMS,
			headers=self._HEADERS,
			content=payload)

	@classmethod
	def _convert_listed_tools(
			cls,
			resp: ListFoundryConnectedToolsResponse,
			input_tools: List[FoundryConnectedTool]) -> Iterable[Tuple[FoundryConnectedTool, FoundryToolDetails]]:
		if resp.error:
			raise resp.error.as_exception()
		if not resp.result:
			return

		tool_map = {(tool.project_connection_id, tool.protocol): tool for tool in input_tools}
		for server in resp.result.servers:
			input_tool = tool_map.get((server.project_connection_id, server.protocol))
			if not input_tool:
				continue

			for tool in server.tools:
				details = FoundryToolDetails(
					name=tool.name,
					description=tool.description,
					input_schema=tool.input_schema,
				)
				yield input_tool, details

	def _build_invoke_tool_request(
			self,
			tool: ResolvedFoundryTool,
			arguments: Dict[str, Any],
			user: Optional[UserInfo],
			agent_name: str) -> HttpRequest:
		if tool.definition.source != FoundryToolSource.CONNECTED:
			raise ToolInvocationError(f"Tool {tool.name} is not a Foundry connected tool.", tool=tool)

		tool_def = cast(FoundryConnectedTool, tool.definition)
		payload: Dict[str, Any] = {
			"toolName": tool.name,
			"arguments": arguments,
			"remoteServer": {
				"projectConnectionId": tool_def.project_connection_id,
				"protocol": tool_def.protocol,
			},
		}
		if user:
			payload["user"] = {
				"objectId": user.object_id,
				"tenantId": user.tenant_id,
			}
		return self._client.post(
			self._invoke_tool_path(agent_name),
			params=self._QUERY_PARAMS,
			headers=self._HEADERS,
			content=payload)

	@classmethod
	def _convert_invoke_result(cls, resp: InvokeFoundryConnectedToolsResponse) -> Any:
		if resp.error:
			raise resp.error.as_exception()
		if not resp.result:
			return None
		return resp.result.value


class FoundryConnectedToolsOperations(BaseFoundryConnectedToolsOperations):
	"""Operations for managing Foundry connected tools."""

	@distributed_trace_async
	async def list_tools(self,
						 tools: List[FoundryConnectedTool],
						 user: Optional[UserInfo],
						 agent_name: str) -> Iterable[Tuple[FoundryConnectedTool, FoundryToolDetails]]:
		"""List connected tools.

		:param tools: List of connected tool definitions.
		:type tools: List[FoundryConnectedTool]
		:param user: User information for the request. Value can be None if running in local.
		:type user: Optional[UserInfo]
		:param agent_name: Name of the agent.
		:type agent_name: str
		:return: An async iterable of tuples containing the tool definition and its details.
		:rtype: AsyncIterable[Tuple[FoundryConnectedTool, FoundryToolDetails]]
		"""
		if not tools:
			return []

		request = self._build_list_tools_request(tools, user, agent_name)
		response = await self._send_request(request)
		async with response:
			json_response = self._extract_response_json(response)
			tools_response = ListFoundryConnectedToolsResponse.model_validate(json_response)
		return self._convert_listed_tools(tools_response, tools)


	@distributed_trace_async
	async def invoke_tool(
		self,
		tool: ResolvedFoundryTool,
		arguments: Dict[str, Any],
		user: Optional[UserInfo],
		agent_name: str) -> Any:
		"""Invoke a connected tool.

		:param tool: Tool descriptor to invoke.
		:type tool: ResolvedFoundryTool
		:param arguments: Input arguments for the tool.
		:type arguments: Mapping[str, Any]
		:param user: User information for the request. Value can be None if running in local.
		:type user: Optional[UserInfo]
		:param agent_name: Name of the agent.
		:type agent_name: str
		:return: Result of the tool invocation.
		:rtype: Any
		"""
		request = self._build_invoke_tool_request(tool, arguments, user, agent_name)
		response = await self._send_request(request)
		async with response:
			json_response = self._extract_response_json(response)
			invoke_response = InvokeFoundryConnectedToolsResponse.model_validate(json_response)
		return self._convert_invoke_result(invoke_response)
	


# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from __future__ import annotations

from abc import ABC
import json
from typing import Any, ClassVar, MutableMapping, Type

from azure.core import AsyncPipelineClient
from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, \
    ResourceNotFoundError, ResourceNotModifiedError, map_error
from azure.core.pipeline.transport import AsyncHttpResponse, HttpRequest

ErrorMapping = MutableMapping[int, Type[HttpResponseError]]


class BaseOperations(ABC):
    DEFAULT_ERROR_MAP: ClassVar[ErrorMapping] = {
        401: ClientAuthenticationError,
        404: ResourceNotFoundError,
        409: ResourceExistsError,
        304: ResourceNotModifiedError,
    }

    def __init__(self, client: AsyncPipelineClient, error_map: ErrorMapping | None = None) -> None:
        self._client = client
        self._error_map = self._prepare_error_map(error_map)

    @classmethod
    def _prepare_error_map(cls, custom_error_map: ErrorMapping | None = None) -> MutableMapping:
        """Prepare error map by merging default and custom error mappings.

        :param custom_error_map: Custom error mappings to merge
        :return: Merged error map
        """
        error_map = cls.DEFAULT_ERROR_MAP
        if custom_error_map:
            error_map = dict(cls.DEFAULT_ERROR_MAP)
            error_map.update(custom_error_map)
        return error_map

    async def _send_request(self, request: HttpRequest, *, stream: bool = False, **kwargs: Any) -> AsyncHttpResponse:
        """Send an HTTP request.

        :param request: HTTP request
        :param stream: Stream to be used for HTTP requests
        :param kwargs: Keyword arguments

        :return: Response object
        """
        response: AsyncHttpResponse = await self._client.send_request(request, stream=stream, **kwargs)
        self._handle_response_error(response)
        return response

    def _handle_response_error(self, response: AsyncHttpResponse) -> None:
        """Handle HTTP response errors.

        :param response: HTTP response to check
        :raises HttpResponseError: If response status is not 200
        """
        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=self._error_map)
            raise HttpResponseError(response=response)
        
    def _extract_response_json(self, response: AsyncHttpResponse) -> Any:
        try:
            payload_text = response.text()
            payload_json = json.loads(payload_text) if payload_text else {}
        except AttributeError:
            payload_bytes = response.body()
            payload_json = json.loads(payload_bytes.decode("utf-8")) if payload_bytes else {}
        return payload_json


# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for _facade.py - testing public function ensure_foundry_tool."""
import pytest

from azure.ai.agentserver.core.tools.runtime._facade import ensure_foundry_tool
from azure.ai.agentserver.core.tools.client._models import (
    FoundryConnectedTool,
    FoundryHostedMcpTool,
    FoundryToolProtocol,
    FoundryToolSource,
)
from azure.ai.agentserver.core.tools._exceptions import InvalidToolFacadeError


class TestEnsureFoundryTool:
    """Tests for ensure_foundry_tool public function."""

    def test_returns_same_instance_when_given_foundry_tool(self, sample_hosted_mcp_tool):
        """Test that passing a FoundryTool returns the same instance."""
        result = ensure_foundry_tool(sample_hosted_mcp_tool)

        assert result is sample_hosted_mcp_tool

    def test_returns_same_instance_for_connected_tool(self, sample_connected_tool):
        """Test that passing a FoundryConnectedTool returns the same instance."""
        result = ensure_foundry_tool(sample_connected_tool)

        assert result is sample_connected_tool

    def test_converts_facade_with_mcp_protocol_to_connected_tool(self):
        """Test that a facade with 'mcp' protocol is converted to FoundryConnectedTool."""
        facade = {
            "type": "mcp",
            "project_connection_id": "my-connection"
        }

        result = ensure_foundry_tool(facade)

        assert isinstance(result, FoundryConnectedTool)
        assert result.protocol == FoundryToolProtocol.MCP
        assert result.project_connection_id == "my-connection"
        assert result.source == FoundryToolSource.CONNECTED

    def test_converts_facade_with_a2a_protocol_to_connected_tool(self):
        """Test that a facade with 'a2a' protocol is converted to FoundryConnectedTool."""
        facade = {
            "type": "a2a",
            "project_connection_id": "my-a2a-connection"
        }

        result = ensure_foundry_tool(facade)

        assert isinstance(result, FoundryConnectedTool)
        assert result.protocol == FoundryToolProtocol.A2A
        assert result.project_connection_id == "my-a2a-connection"

    def test_converts_facade_with_unknown_type_to_hosted_mcp_tool(self):
        """Test that a facade with unknown type is converted to FoundryHostedMcpTool."""
        facade = {
            "type": "my_custom_tool",
            "some_config": "value123",
            "another_config": True
        }

        result = ensure_foundry_tool(facade)

        assert isinstance(result, FoundryHostedMcpTool)
        assert result.name == "my_custom_tool"
        assert result.configuration == {"some_config": "value123", "another_config": True}
        assert result.source == FoundryToolSource.HOSTED_MCP

    def test_raises_error_when_type_is_missing(self):
        """Test that InvalidToolFacadeError is raised when 'type' is missing."""
        facade = {"project_connection_id": "my-connection"}

        with pytest.raises(InvalidToolFacadeError) as exc_info:
            ensure_foundry_tool(facade)

        assert "type" in str(exc_info.value).lower()

    def test_raises_error_when_type_is_empty_string(self):
        """Test that InvalidToolFacadeError is raised when 'type' is empty string."""
        facade = {"type": "", "project_connection_id": "my-connection"}

        with pytest.raises(InvalidToolFacadeError) as exc_info:
            ensure_foundry_tool(facade)

        assert "type" in str(exc_info.value).lower()

    def test_raises_error_when_type_is_not_string(self):
        """Test that InvalidToolFacadeError is raised when 'type' is not a string."""
        facade = {"type": 123, "project_connection_id": "my-connection"}

        with pytest.raises(InvalidToolFacadeError) as exc_info:
            ensure_foundry_tool(facade)

        assert "type" in str(exc_info.value).lower()

    def test_raises_error_when_mcp_protocol_missing_connection_id(self):
        """Test that InvalidToolFacadeError is raised when mcp protocol is missing project_connection_id."""
        facade = {"type": "mcp"}

        with pytest.raises(InvalidToolFacadeError) as exc_info:
            ensure_foundry_tool(facade)

        assert "project_connection_id" in str(exc_info.value)

    def test_raises_error_when_a2a_protocol_has_empty_connection_id(self):
        """Test that InvalidToolFacadeError is raised when a2a protocol has empty project_connection_id."""
        facade = {"type": "a2a", "project_connection_id": ""}

        with pytest.raises(InvalidToolFacadeError) as exc_info:
            ensure_foundry_tool(facade)

        assert "project_connection_id" in str(exc_info.value)

    def test_parses_resource_id_format_connection_id(self):
        """Test that resource ID format project_connection_id is parsed correctly."""
        resource_id = (
            "/subscriptions/sub-123/resourceGroups/rg-test/providers/"
            "Microsoft.CognitiveServices/accounts/acc-test/projects/proj-test/connections/my-conn-name"
        )
        facade = {
            "type": "mcp",
            "project_connection_id": resource_id
        }

        result = ensure_foundry_tool(facade)

        assert isinstance(result, FoundryConnectedTool)
        assert result.project_connection_id == "my-conn-name"

    def test_raises_error_for_invalid_resource_id_format(self):
        """Test that InvalidToolFacadeError is raised for invalid resource ID format."""
        invalid_resource_id = "/subscriptions/sub-123/invalid/path"
        facade = {
            "type": "mcp",
            "project_connection_id": invalid_resource_id
        }

        with pytest.raises(InvalidToolFacadeError) as exc_info:
            ensure_foundry_tool(facade)

        assert "Invalid resource ID format" in str(exc_info.value)

    def test_uses_simple_connection_name_as_is(self):
        """Test that simple connection name is used as-is without parsing."""
        facade = {
            "type": "mcp",
            "project_connection_id": "simple-connection-name"
        }

        result = ensure_foundry_tool(facade)

        assert isinstance(result, FoundryConnectedTool)
        assert result.project_connection_id == "simple-connection-name"

    def test_original_facade_not_modified(self):
        """Test that the original facade dictionary is not modified."""
        facade = {
            "type": "my_tool",
            "config_key": "config_value"
        }
        original_facade = facade.copy()

        ensure_foundry_tool(facade)

        assert facade == original_facade

    def test_hosted_mcp_tool_with_no_extra_configuration(self):
        """Test that hosted MCP tool works with no extra configuration."""
        facade = {"type": "simple_tool"}

        result = ensure_foundry_tool(facade)

        assert isinstance(result, FoundryHostedMcpTool)
        assert result.name == "simple_tool"
        assert result.configuration == {}



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for FoundryToolClient - testing only public methods."""
import pytest
from unittest.mock import AsyncMock, MagicMock, patch

from azure.ai.agentserver.core.tools.client._client import FoundryToolClient
from azure.ai.agentserver.core.tools.client._models import (
    FoundryToolDetails,
    FoundryToolSource,
    ResolvedFoundryTool,
)
from azure.ai.agentserver.core.tools._exceptions import ToolInvocationError

from ..conftest import create_mock_http_response


class TestFoundryToolClientInit:
    """Tests for FoundryToolClient.__init__ public method."""

    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    def test_init_with_valid_endpoint_and_credential(self, mock_pipeline_client_class, mock_credential):
        """Test client can be initialized with valid endpoint and credential."""
        endpoint = "https://fake-project-endpoint.site"

        client = FoundryToolClient(endpoint, mock_credential)

        # Verify client was created with correct base_url
        call_kwargs = mock_pipeline_client_class.call_args
        assert call_kwargs[1]["base_url"] == endpoint
        assert client is not None


class TestFoundryToolClientListTools:
    """Tests for FoundryToolClient.list_tools public method."""

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_list_tools_empty_collection_returns_empty_list(
        self,
        mock_pipeline_client_class,
        mock_credential
    ):
        """Test list_tools returns empty list when given empty collection."""
        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)

        result = await client.list_tools([], agent_name="test-agent")

        assert result == []

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_list_tools_with_single_mcp_tool_returns_resolved_tools(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_hosted_mcp_tool
    ):
        """Test list_tools with a single MCP tool returns resolved tools."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        # Mock HTTP response for MCP tools listing
        response_data = {
            "jsonrpc": "2.0",
            "id": 1,
            "result": {
                "tools": [
                    {
                        "name": sample_hosted_mcp_tool.name,
                        "description": "Test MCP tool description",
                        "inputSchema": {"type": "object", "properties": {}}
                    }
                ]
            }
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.list_tools([sample_hosted_mcp_tool], agent_name="test-agent")

        assert len(result) == 1
        assert isinstance(result[0], ResolvedFoundryTool)
        assert result[0].name == sample_hosted_mcp_tool.name
        assert result[0].source == FoundryToolSource.HOSTED_MCP

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_list_tools_with_single_connected_tool_returns_resolved_tools(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_connected_tool,
        sample_user_info
    ):
        """Test list_tools with a single connected tool returns resolved tools."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        # Mock HTTP response for connected tools listing
        response_data = {
            "tools": [
                {
                    "remoteServer": {
                        "protocol": sample_connected_tool.protocol,
                        "projectConnectionId": sample_connected_tool.project_connection_id
                    },
                    "manifest": [
                        {
                            "name": "connected_test_tool",
                            "description": "Test connected tool",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                }
            ]
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.list_tools(
            [sample_connected_tool],
            agent_name="test-agent",
            user=sample_user_info
        )

        assert len(result) == 1
        assert isinstance(result[0], ResolvedFoundryTool)
        assert result[0].name == "connected_test_tool"
        assert result[0].source == FoundryToolSource.CONNECTED

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_list_tools_with_mixed_tool_types_returns_all_resolved(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_hosted_mcp_tool,
        sample_connected_tool,
        sample_user_info
    ):
        """Test list_tools with both MCP and connected tools returns all resolved tools."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        # We need to return different responses based on the request
        mcp_response_data = {
            "jsonrpc": "2.0",
            "id": 1,
            "result": {
                "tools": [
                    {
                        "name": sample_hosted_mcp_tool.name,
                        "description": "MCP tool",
                        "inputSchema": {"type": "object", "properties": {}}
                    }
                ]
            }
        }
        connected_response_data = {
            "tools": [
                {
                    "remoteServer": {
                        "protocol": sample_connected_tool.protocol,
                        "projectConnectionId": sample_connected_tool.project_connection_id
                    },
                    "manifest": [
                        {
                            "name": "connected_tool",
                            "description": "Connected tool",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                }
            ]
        }

        # Mock to return different responses for different requests
        mock_client_instance.send_request.side_effect = [
            create_mock_http_response(200, mcp_response_data),
            create_mock_http_response(200, connected_response_data)
        ]
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.list_tools(
            [sample_hosted_mcp_tool, sample_connected_tool],
            agent_name="test-agent",
            user=sample_user_info
        )

        assert len(result) == 2
        sources = {tool.source for tool in result}
        assert FoundryToolSource.HOSTED_MCP in sources
        assert FoundryToolSource.CONNECTED in sources

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_list_tools_filters_unlisted_mcp_tools(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_hosted_mcp_tool
    ):
        """Test list_tools only returns tools that are in the allowed list."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        # Server returns more tools than requested
        response_data = {
            "jsonrpc": "2.0",
            "id": 1,
            "result": {
                "tools": [
                    {
                        "name": sample_hosted_mcp_tool.name,
                        "description": "Requested tool",
                        "inputSchema": {"type": "object", "properties": {}}
                    },
                    {
                        "name": "unrequested_tool",
                        "description": "This tool was not requested",
                        "inputSchema": {"type": "object", "properties": {}}
                    }
                ]
            }
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.list_tools([sample_hosted_mcp_tool], agent_name="test-agent")

        # Should only return the requested tool
        assert len(result) == 1
        assert result[0].name == sample_hosted_mcp_tool.name


class TestFoundryToolClientListToolsDetails:
    """Tests for FoundryToolClient.list_tools_details public method."""

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_list_tools_details_returns_mapping_structure(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_hosted_mcp_tool
    ):
        """Test list_tools_details returns correct mapping structure."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        response_data = {
            "jsonrpc": "2.0",
            "id": 1,
            "result": {
                "tools": [
                    {
                        "name": sample_hosted_mcp_tool.name,
                        "description": "Test tool",
                        "inputSchema": {"type": "object", "properties": {}}
                    }
                ]
            }
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.list_tools_details([sample_hosted_mcp_tool], agent_name="test-agent")

        assert isinstance(result, dict)
        assert sample_hosted_mcp_tool.id in result
        assert len(result[sample_hosted_mcp_tool.id]) == 1
        assert isinstance(result[sample_hosted_mcp_tool.id][0], FoundryToolDetails)

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_list_tools_details_groups_multiple_tools_by_definition(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_hosted_mcp_tool
    ):
        """Test list_tools_details groups multiple tools from same source by definition ID."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        # Server returns multiple tools for the same MCP source
        response_data = {
            "jsonrpc": "2.0",
            "id": 1,
            "result": {
                "tools": [
                    {
                        "name": sample_hosted_mcp_tool.name,
                        "description": "Tool variant 1",
                        "inputSchema": {"type": "object", "properties": {}}
                    }
                ]
            }
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.list_tools_details([sample_hosted_mcp_tool], agent_name="test-agent")

        # All tools should be grouped under the same definition ID
        assert sample_hosted_mcp_tool.id in result


class TestFoundryToolClientInvokeTool:
    """Tests for FoundryToolClient.invoke_tool public method."""

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_invoke_mcp_tool_returns_result(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_resolved_mcp_tool
    ):
        """Test invoke_tool with MCP tool returns the invocation result."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        expected_result = {"result": {"content": [{"text": "Hello World"}]}}
        mock_response = create_mock_http_response(200, expected_result)
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.invoke_tool(
            sample_resolved_mcp_tool,
            arguments={"input": "test"},
            agent_name="test-agent"
        )

        assert result == expected_result

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_invoke_connected_tool_returns_result(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_resolved_connected_tool,
        sample_user_info
    ):
        """Test invoke_tool with connected tool returns the invocation result."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        expected_value = {"output": "Connected tool result"}
        response_data = {"toolResult": expected_value}
        mock_response = create_mock_http_response(200, response_data)
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        result = await client.invoke_tool(
            sample_resolved_connected_tool,
            arguments={"input": "test"},
            agent_name="test-agent",
            user=sample_user_info
        )

        assert result == expected_value

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_invoke_tool_with_complex_arguments(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_resolved_mcp_tool
    ):
        """Test invoke_tool correctly passes complex arguments."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        mock_response = create_mock_http_response(200, {"result": "success"})
        mock_client_instance.send_request.return_value = mock_response
        mock_client_instance.post.return_value = MagicMock()

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        complex_args = {
            "string_param": "value",
            "number_param": 42,
            "bool_param": True,
            "list_param": [1, 2, 3],
            "nested_param": {"key": "value"}
        }

        result = await client.invoke_tool(
            sample_resolved_mcp_tool,
            arguments=complex_args,
            agent_name="test-agent"
        )

        # Verify request was made
        mock_client_instance.send_request.assert_called_once()

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_invoke_tool_with_unsupported_source_raises_error(
        self,
        mock_pipeline_client_class,
        mock_credential,
        sample_tool_details
    ):
        """Test invoke_tool raises ToolInvocationError for unsupported tool source."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        # Create a mock tool with unsupported source
        mock_definition = MagicMock()
        mock_definition.source = "unsupported_source"
        mock_tool = MagicMock(spec=ResolvedFoundryTool)
        mock_tool.definition = mock_definition
        mock_tool.source = "unsupported_source"
        mock_tool.details = sample_tool_details

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)

        with pytest.raises(ToolInvocationError) as exc_info:
            await client.invoke_tool(
                mock_tool,
                arguments={"input": "test"},
                agent_name="test-agent"
            )

        assert "Unsupported tool source" in str(exc_info.value)


class TestFoundryToolClientClose:
    """Tests for FoundryToolClient.close public method."""

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_close_closes_underlying_client(
        self,
        mock_pipeline_client_class,
        mock_credential
    ):
        """Test close() properly closes the underlying HTTP client."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        client = FoundryToolClient("https://fake-project-endpoint.site", mock_credential)
        await client.close()

        mock_client_instance.close.assert_called_once()


class TestFoundryToolClientContextManager:
    """Tests for FoundryToolClient async context manager protocol."""

    @pytest.mark.asyncio
    @patch("azure.ai.agentserver.core.tools.client._client.AsyncPipelineClient")
    async def test_async_context_manager_enters_and_exits(
        self,
        mock_pipeline_client_class,
        mock_credential
    ):
        """Test client can be used as async context manager."""
        mock_client_instance = AsyncMock()
        mock_pipeline_client_class.return_value = mock_client_instance

        async with FoundryToolClient("https://fake-project-endpoint.site", mock_credential) as client:
            assert client is not None
            mock_client_instance.__aenter__.assert_called_once()

        mock_client_instance.__aexit__.assert_called_once()




# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for FoundryConnectedToolsOperations - testing only public methods."""
import pytest
from unittest.mock import AsyncMock, MagicMock

from azure.ai.agentserver.core.tools.client._models import (
    FoundryConnectedTool,
    FoundryToolDetails,
)
from azure.ai.agentserver.core.tools.client.operations._foundry_connected_tools import (
    FoundryConnectedToolsOperations,
)
from azure.ai.agentserver.core.tools._exceptions import OAuthConsentRequiredError, ToolInvocationError

from ...conftest import create_mock_http_response


class TestFoundryConnectedToolsOperationsListTools:
    """Tests for FoundryConnectedToolsOperations.list_tools public method."""

    @pytest.mark.asyncio
    async def test_list_tools_with_empty_list_returns_empty(self):
        """Test list_tools returns empty when tools list is empty."""
        mock_client = AsyncMock()
        ops = FoundryConnectedToolsOperations(mock_client)

        result = await ops.list_tools([], None, "test-agent")

        assert result == []
        # Should not make any HTTP request
        mock_client.send_request.assert_not_called()

    @pytest.mark.asyncio
    async def test_list_tools_returns_tools_from_server(
        self,
        sample_connected_tool,
        sample_user_info
    ):
        """Test list_tools returns tools from server response."""
        mock_client = AsyncMock()

        response_data = {
            "tools": [
                {
                    "remoteServer": {
                        "protocol": sample_connected_tool.protocol,
                        "projectConnectionId": sample_connected_tool.project_connection_id
                    },
                    "manifest": [
                        {
                            "name": "remote_tool",
                            "description": "A remote connected tool",
                            "parameters": {
                                "type": "object",
                                "properties": {
                                    "input": {"type": "string"}
                                }
                            }
                        }
                    ]
                }
            ]
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = list(await ops.list_tools([sample_connected_tool], sample_user_info, "test-agent"))

        assert len(result) == 1
        definition, details = result[0]
        assert definition == sample_connected_tool
        assert isinstance(details, FoundryToolDetails)
        assert details.name == "remote_tool"
        assert details.description == "A remote connected tool"

    @pytest.mark.asyncio
    async def test_list_tools_without_user_info(self, sample_connected_tool):
        """Test list_tools works without user info (local execution)."""
        mock_client = AsyncMock()

        response_data = {
            "tools": [
                {
                    "remoteServer": {
                        "protocol": sample_connected_tool.protocol,
                        "projectConnectionId": sample_connected_tool.project_connection_id
                    },
                    "manifest": [
                        {
                            "name": "tool_no_user",
                            "description": "Tool without user",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                }
            ]
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = list(await ops.list_tools([sample_connected_tool], None, "test-agent"))

        assert len(result) == 1
        assert result[0][1].name == "tool_no_user"

    @pytest.mark.asyncio
    async def test_list_tools_with_multiple_connections(self, sample_user_info):
        """Test list_tools with multiple connected tool definitions."""
        mock_client = AsyncMock()

        tool1 = FoundryConnectedTool(protocol="mcp", project_connection_id="conn-1")
        tool2 = FoundryConnectedTool(protocol="a2a", project_connection_id="conn-2")

        response_data = {
            "tools": [
                {
                    "remoteServer": {
                        "protocol": "mcp",
                        "projectConnectionId": "conn-1"
                    },
                    "manifest": [
                        {
                            "name": "tool_from_conn1",
                            "description": "From connection 1",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                },
                {
                    "remoteServer": {
                        "protocol": "a2a",
                        "projectConnectionId": "conn-2"
                    },
                    "manifest": [
                        {
                            "name": "tool_from_conn2",
                            "description": "From connection 2",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                }
            ]
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = list(await ops.list_tools([tool1, tool2], sample_user_info, "test-agent"))

        assert len(result) == 2
        names = {r[1].name for r in result}
        assert names == {"tool_from_conn1", "tool_from_conn2"}

    @pytest.mark.asyncio
    async def test_list_tools_filters_by_connection_id(self, sample_user_info):
        """Test list_tools only returns tools from requested connections."""
        mock_client = AsyncMock()

        requested_tool = FoundryConnectedTool(protocol="mcp", project_connection_id="requested-conn")

        # Server returns tools from multiple connections, but we only requested one
        response_data = {
            "tools": [
                {
                    "remoteServer": {
                        "protocol": "mcp",
                        "projectConnectionId": "requested-conn"
                    },
                    "manifest": [
                        {
                            "name": "requested_tool",
                            "description": "Requested",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                },
                {
                    "remoteServer": {
                        "protocol": "mcp",
                        "projectConnectionId": "unrequested-conn"
                    },
                    "manifest": [
                        {
                            "name": "unrequested_tool",
                            "description": "Not requested",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                }
            ]
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = list(await ops.list_tools([requested_tool], sample_user_info, "test-agent"))

        # Should only return tools from requested connection
        assert len(result) == 1
        assert result[0][1].name == "requested_tool"

    @pytest.mark.asyncio
    async def test_list_tools_multiple_tools_per_connection(
        self,
        sample_connected_tool,
        sample_user_info
    ):
        """Test list_tools returns multiple tools from same connection."""
        mock_client = AsyncMock()

        response_data = {
            "tools": [
                {
                    "remoteServer": {
                        "protocol": sample_connected_tool.protocol,
                        "projectConnectionId": sample_connected_tool.project_connection_id
                    },
                    "manifest": [
                        {
                            "name": "tool_one",
                            "description": "First tool",
                            "parameters": {"type": "object", "properties": {}}
                        },
                        {
                            "name": "tool_two",
                            "description": "Second tool",
                            "parameters": {"type": "object", "properties": {}}
                        },
                        {
                            "name": "tool_three",
                            "description": "Third tool",
                            "parameters": {"type": "object", "properties": {}}
                        }
                    ]
                }
            ]
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = list(await ops.list_tools([sample_connected_tool], sample_user_info, "test-agent"))

        assert len(result) == 3
        names = {r[1].name for r in result}
        assert names == {"tool_one", "tool_two", "tool_three"}

    @pytest.mark.asyncio
    async def test_list_tools_raises_oauth_consent_error(
        self,
        sample_connected_tool,
        sample_user_info
    ):
        """Test list_tools raises OAuthConsentRequiredError when consent needed."""
        mock_client = AsyncMock()

        response_data = {
            "type": "OAuthConsentRequired",
            "toolResult": {
                "consentUrl": "https://login.microsoftonline.com/consent",
                "message": "User consent is required to access this resource",
                "projectConnectionId": sample_connected_tool.project_connection_id
            }
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)

        with pytest.raises(OAuthConsentRequiredError) as exc_info:
            list(await ops.list_tools([sample_connected_tool], sample_user_info, "test-agent"))

        assert exc_info.value.consent_url == "https://login.microsoftonline.com/consent"
        assert "consent" in exc_info.value.message.lower()


class TestFoundryConnectedToolsOperationsInvokeTool:
    """Tests for FoundryConnectedToolsOperations.invoke_tool public method."""

    @pytest.mark.asyncio
    async def test_invoke_tool_returns_result_value(
        self,
        sample_resolved_connected_tool,
        sample_user_info
    ):
        """Test invoke_tool returns the result value from server."""
        mock_client = AsyncMock()

        expected_result = {"data": "some output", "status": "success"}
        response_data = {"toolResult": expected_result}
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = await ops.invoke_tool(
            sample_resolved_connected_tool,
            {"input": "test"},
            sample_user_info,
            "test-agent"
        )

        assert result == expected_result

    @pytest.mark.asyncio
    async def test_invoke_tool_without_user_info(self, sample_resolved_connected_tool):
        """Test invoke_tool works without user info (local execution)."""
        mock_client = AsyncMock()

        response_data = {"toolResult": "local result"}
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = await ops.invoke_tool(
            sample_resolved_connected_tool,
            {},
            None,  # No user info
            "test-agent"
        )

        assert result == "local result"

    @pytest.mark.asyncio
    async def test_invoke_tool_with_complex_arguments(
        self,
        sample_resolved_connected_tool,
        sample_user_info
    ):
        """Test invoke_tool handles complex nested arguments."""
        mock_client = AsyncMock()

        response_data = {"toolResult": "processed"}
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        complex_args = {
            "query": "search term",
            "filters": {
                "date_range": {"start": "2025-01-01", "end": "2025-12-31"},
                "categories": ["A", "B", "C"]
            },
            "limit": 50
        }

        result = await ops.invoke_tool(
            sample_resolved_connected_tool,
            complex_args,
            sample_user_info,
            "test-agent"
        )

        assert result == "processed"
        mock_client.send_request.assert_called_once()

    @pytest.mark.asyncio
    async def test_invoke_tool_returns_none_for_empty_result(
        self,
        sample_resolved_connected_tool,
        sample_user_info
    ):
        """Test invoke_tool returns None when server returns no result."""
        mock_client = AsyncMock()

        # Server returns empty response (no toolResult)
        response_data = {
            "toolResult": None
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)
        result = await ops.invoke_tool(
            sample_resolved_connected_tool,
            {},
            sample_user_info,
            "test-agent"
        )

        assert result is None

    @pytest.mark.asyncio
    async def test_invoke_tool_with_mcp_tool_raises_error(
        self,
        sample_resolved_mcp_tool,
        sample_user_info
    ):
        """Test invoke_tool raises ToolInvocationError for non-connected tool."""
        mock_client = AsyncMock()
        ops = FoundryConnectedToolsOperations(mock_client)

        with pytest.raises(ToolInvocationError) as exc_info:
            await ops.invoke_tool(
                sample_resolved_mcp_tool,
                {},
                sample_user_info,
                "test-agent"
            )

        assert "not a Foundry connected tool" in str(exc_info.value)
        # Should not make any HTTP request
        mock_client.send_request.assert_not_called()

    @pytest.mark.asyncio
    async def test_invoke_tool_raises_oauth_consent_error(
        self,
        sample_resolved_connected_tool,
        sample_user_info
    ):
        """Test invoke_tool raises OAuthConsentRequiredError when consent needed."""
        mock_client = AsyncMock()

        response_data = {
            "type": "OAuthConsentRequired",
            "toolResult": {
                "consentUrl": "https://login.microsoftonline.com/oauth/consent",
                "message": "Please provide consent to continue",
                "projectConnectionId": "test-connection-id"
            }
        }
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)

        with pytest.raises(OAuthConsentRequiredError) as exc_info:
            await ops.invoke_tool(
                sample_resolved_connected_tool,
                {"input": "test"},
                sample_user_info,
                "test-agent"
            )

        assert "https://login.microsoftonline.com/oauth/consent" in exc_info.value.consent_url

    @pytest.mark.asyncio
    async def test_invoke_tool_with_different_agent_names(
        self,
        sample_resolved_connected_tool,
        sample_user_info
    ):
        """Test invoke_tool uses correct agent name in request."""
        mock_client = AsyncMock()

        response_data = {"toolResult": "result"}
        mock_response = create_mock_http_response(200, response_data)
        mock_client.send_request.return_value = mock_response
        mock_client.post.return_value = MagicMock()

        ops = FoundryConnectedToolsOperations(mock_client)

        # Invoke with different agent names
        for agent_name in ["agent-1", "my-custom-agent", "production-agent"]:
            await ops.invoke_tool(
                sample_resolved_connected_tool,
                {},
                sample_user_info,
                agent_name
            )

            # Verify the correct path was used
            call_args = mock_client.post.call_args
            assert agent_name in call_args[0][0]




# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for _resolver.py - testing public methods of DefaultFoundryToolInvocationResolver."""
import pytest
from unittest.mock import AsyncMock, MagicMock

from azure.ai.agentserver.core.tools.runtime._resolver import DefaultFoundryToolInvocationResolver
from azure.ai.agentserver.core.tools.runtime._invoker import DefaultFoundryToolInvoker
from azure.ai.agentserver.core.tools._exceptions import UnableToResolveToolInvocationError
from azure.ai.agentserver.core.tools.client._models import (
    FoundryConnectedTool,
    FoundryHostedMcpTool,
)


class TestDefaultFoundryToolInvocationResolverResolve:
    """Tests for DefaultFoundryToolInvocationResolver.resolve method."""

    @pytest.fixture
    def mock_catalog(self, sample_resolved_mcp_tool):
        """Create a mock FoundryToolCatalog."""
        catalog = AsyncMock()
        catalog.get = AsyncMock(return_value=sample_resolved_mcp_tool)
        catalog.list = AsyncMock(return_value=[sample_resolved_mcp_tool])
        return catalog

    @pytest.mark.asyncio
    async def test_resolve_with_resolved_tool_returns_invoker_directly(
        self,
        mock_catalog,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_resolved_mcp_tool
    ):
        """Test resolve returns invoker directly when given ResolvedFoundryTool."""
        resolver = DefaultFoundryToolInvocationResolver(
            catalog=mock_catalog,
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        invoker = await resolver.resolve(sample_resolved_mcp_tool)

        assert isinstance(invoker, DefaultFoundryToolInvoker)
        assert invoker.resolved_tool is sample_resolved_mcp_tool
        # Catalog should not be called when ResolvedFoundryTool is passed
        mock_catalog.get.assert_not_called()

    @pytest.mark.asyncio
    async def test_resolve_with_foundry_tool_uses_catalog(
        self,
        mock_catalog,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool,
        sample_resolved_mcp_tool
    ):
        """Test resolve uses catalog to resolve FoundryTool."""
        mock_catalog.get = AsyncMock(return_value=sample_resolved_mcp_tool)

        resolver = DefaultFoundryToolInvocationResolver(
            catalog=mock_catalog,
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        invoker = await resolver.resolve(sample_hosted_mcp_tool)

        assert isinstance(invoker, DefaultFoundryToolInvoker)
        mock_catalog.get.assert_called_once_with(sample_hosted_mcp_tool)

    @pytest.mark.asyncio
    async def test_resolve_with_facade_dict_uses_catalog(
        self,
        mock_catalog,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_resolved_connected_tool
    ):
        """Test resolve converts facade dict and uses catalog."""
        mock_catalog.get = AsyncMock(return_value=sample_resolved_connected_tool)
        facade = {
            "type": "mcp",
            "project_connection_id": "test-connection"
        }

        resolver = DefaultFoundryToolInvocationResolver(
            catalog=mock_catalog,
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        invoker = await resolver.resolve(facade)

        assert isinstance(invoker, DefaultFoundryToolInvoker)
        mock_catalog.get.assert_called_once()
        # Verify the facade was converted to FoundryConnectedTool
        call_arg = mock_catalog.get.call_args[0][0]
        assert isinstance(call_arg, FoundryConnectedTool)

    @pytest.mark.asyncio
    async def test_resolve_raises_error_when_tool_not_found_in_catalog(
        self,
        mock_catalog,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_hosted_mcp_tool
    ):
        """Test resolve raises UnableToResolveToolInvocationError when catalog returns None."""
        mock_catalog.get = AsyncMock(return_value=None)

        resolver = DefaultFoundryToolInvocationResolver(
            catalog=mock_catalog,
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        with pytest.raises(UnableToResolveToolInvocationError) as exc_info:
            await resolver.resolve(sample_hosted_mcp_tool)

        assert exc_info.value.tool is sample_hosted_mcp_tool
        assert "Unable to resolve tool" in str(exc_info.value)

    @pytest.mark.asyncio
    async def test_resolve_with_hosted_mcp_facade(
        self,
        mock_catalog,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_resolved_mcp_tool
    ):
        """Test resolve with hosted MCP facade (unknown type becomes FoundryHostedMcpTool)."""
        mock_catalog.get = AsyncMock(return_value=sample_resolved_mcp_tool)
        facade = {
            "type": "custom_mcp_tool",
            "config_key": "config_value"
        }

        resolver = DefaultFoundryToolInvocationResolver(
            catalog=mock_catalog,
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        invoker = await resolver.resolve(facade)

        assert isinstance(invoker, DefaultFoundryToolInvoker)
        # Verify the facade was converted to FoundryHostedMcpTool
        call_arg = mock_catalog.get.call_args[0][0]
        assert isinstance(call_arg, FoundryHostedMcpTool)
        assert call_arg.name == "custom_mcp_tool"

    @pytest.mark.asyncio
    async def test_resolve_returns_invoker_with_correct_agent_name(
        self,
        mock_catalog,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_resolved_mcp_tool
    ):
        """Test resolve creates invoker with the correct agent name."""
        resolver = DefaultFoundryToolInvocationResolver(
            catalog=mock_catalog,
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="custom-agent-name"
        )

        invoker = await resolver.resolve(sample_resolved_mcp_tool)

        # Verify invoker was created with correct agent name by checking internal state
        assert invoker._agent_name == "custom-agent-name"

    @pytest.mark.asyncio
    async def test_resolve_with_connected_tool_directly(
        self,
        mock_catalog,
        mock_foundry_tool_client,
        mock_user_provider,
        sample_connected_tool,
        sample_resolved_connected_tool
    ):
        """Test resolve with FoundryConnectedTool directly."""
        mock_catalog.get = AsyncMock(return_value=sample_resolved_connected_tool)

        resolver = DefaultFoundryToolInvocationResolver(
            catalog=mock_catalog,
            client=mock_foundry_tool_client,
            user_provider=mock_user_provider,
            agent_name="test-agent"
        )

        invoker = await resolver.resolve(sample_connected_tool)

        assert isinstance(invoker, DefaultFoundryToolInvoker)
        mock_catalog.get.assert_called_once_with(sample_connected_tool)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
import asyncio  # pylint: disable=C4763
from abc import ABC, abstractmethod
from typing import Any, Awaitable, Collection, List, Mapping, MutableMapping, Optional, Union

from cachetools import TTLCache  # type: ignore[import-untyped]

from ._facade import FoundryToolLike, ensure_foundry_tool
from ._user import UserProvider
from ..client._client import FoundryToolClient
from ..client._models import FoundryTool, FoundryToolDetails, FoundryToolSource, ResolvedFoundryTool, UserInfo


class FoundryToolCatalog(ABC):
    """Base class for Foundry tool catalogs."""
    def __init__(self, user_provider: UserProvider):
        self._user_provider = user_provider

    async def get(self, tool: FoundryToolLike) -> Optional[ResolvedFoundryTool]:
        """Gets a Foundry tool by its definition.

        :param tool: The Foundry tool to resolve.
        :type tool: FoundryToolLike
        :return: The resolved Foundry tool.
        :rtype: Optional[ResolvedFoundryTool]
        """
        tools = await self.list([tool])
        return tools[0] if tools else None

    @abstractmethod
    async def list(self, tools: List[FoundryToolLike]) -> List[ResolvedFoundryTool]:
        """Lists all available Foundry tools.

        :param tools: The list of Foundry tools to resolve.
        :type tools: List[FoundryToolLike]
        :return: A list of resolved Foundry tools.
        :rtype: List[ResolvedFoundryTool]
        """
        raise NotImplementedError


_CachedValueType = Union[Awaitable[List[FoundryToolDetails]], List[FoundryToolDetails]]


class CachedFoundryToolCatalog(FoundryToolCatalog, ABC):
    """Cached implementation of FoundryToolCatalog with concurrency-safe caching."""

    def __init__(self, user_provider: UserProvider):
        super().__init__(user_provider)
        self._cache: MutableMapping[Any, _CachedValueType] = self._create_cache()

    def _create_cache(self) -> MutableMapping[Any, _CachedValueType]:
        return TTLCache(maxsize=1024, ttl=600)

    def _get_key(self, user: Optional[UserInfo], tool: FoundryTool) -> Any:
        if tool.source is FoundryToolSource.HOSTED_MCP:
            return tool.id
        return user, tool.id

    async def list(self, tools: List[FoundryToolLike]) -> List[ResolvedFoundryTool]:
        user = await self._user_provider.get_user()
        foundry_tools = {}
        tools_to_fetch = {}
        fetching_tasks = []
        for t in tools:
            tool = ensure_foundry_tool(t)
            key = self._get_key(user, tool)
            foundry_tools[key] = tool
            if key not in self._cache:
                tools_to_fetch[key] = tool
            elif (task := self._cache[key]) and isinstance(task, Awaitable):
                fetching_tasks.append(task)

        # for tools that are not being listed, create a batch task, convert to per-tool resolving tasks, and cache them
        if tools_to_fetch:
            # Awaitable[Mapping[str, List[FoundryToolDetails]]]
            fetched_tools = asyncio.create_task(self._fetch_tools(tools_to_fetch.values(), user))

            for k, t in tools_to_fetch.items():
                # safe to write cache since it's the only runner in this event loop
                task = asyncio.create_task(self._per_tool_fetching_task(k, t, fetched_tools))
                self._cache[k] = task
                fetching_tasks.append(task)

        try:
            # now we have every tool associated with a task
            if fetching_tasks:
                await asyncio.gather(*fetching_tasks)
        except:
            # exception can only be caused by fetching tasks, remove them from cache
            for k, _ in tools_to_fetch.items():
                if k in self._cache:
                    del self._cache[k]
            raise

        resolved_tools = []
        for key, tool in foundry_tools.items():
            # this acts like a lock - every task of the same tool waits for the same underlying fetch
            task_or_value = self._cache[key]
            details_list = (await task_or_value) if isinstance(task_or_value, Awaitable) else task_or_value
            for details in details_list:
                resolved_tools.append(
                    ResolvedFoundryTool(
                        definition=tool,
                        details=details
                    )
                )

        return resolved_tools

    async def _per_tool_fetching_task(
            self,
            cache_key: Any,
            tool: FoundryTool,
            fetching: Awaitable[Mapping[str, List[FoundryToolDetails]]]
    ) -> List[FoundryToolDetails]:
        details = await fetching
        details_list = details.get(tool.id, [])
        # replace the task in cache with the actual value to optimize memory usage
        self._cache[cache_key] = details_list
        return details_list

    @abstractmethod
    async def _fetch_tools(self,
                           tools: Collection[FoundryTool],
                           user: Optional[UserInfo]) -> Mapping[str, List[FoundryToolDetails]]:
        raise NotImplementedError


class DefaultFoundryToolCatalog(CachedFoundryToolCatalog):
    """Default implementation of FoundryToolCatalog."""

    def __init__(self, client: FoundryToolClient, user_provider: UserProvider, agent_name: str):
        super().__init__(user_provider)
        self._client = client
        self._agent_name = agent_name

    async def _fetch_tools(self,
                           tools: Collection[FoundryTool],
                           user: Optional[UserInfo]) -> Mapping[str, List[FoundryToolDetails]]:
        return await self._client.list_tools_details(tools, self._agent_name, user)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
class Constants:
    # well-known environment variables
    AZURE_AI_PROJECT_ENDPOINT = "AZURE_AI_PROJECT_ENDPOINT"
    AGENT_ID = "AGENT_ID"
    AGENT_NAME = "AGENT_NAME"
    AGENT_PROJECT_RESOURCE_ID = "AGENT_PROJECT_NAME"
    OTEL_EXPORTER_ENDPOINT = "OTEL_EXPORTER_ENDPOINT"
    OTEL_EXPORTER_OTLP_PROTOCOL = "OTEL_EXPORTER_OTLP_PROTOCOL"
    AGENT_LOG_LEVEL = "AGENT_LOG_LEVEL"
    AGENT_DEBUG_ERRORS = "AGENT_DEBUG_ERRORS"
    ENABLE_APPLICATION_INSIGHTS_LOGGER = "AGENT_APP_INSIGHTS_ENABLED"
    AZURE_AI_WORKSPACE_ENDPOINT = "AZURE_AI_WORKSPACE_ENDPOINT"
    AZURE_AI_TOOLS_ENDPOINT = "AZURE_AI_TOOLS_ENDPOINT"



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from azure.core.configuration import Configuration
from azure.core.credentials_async import AsyncTokenCredential
from azure.core.pipeline import policies

from ...application._package_metadata import get_current_app


class FoundryToolClientConfiguration(Configuration):  # pylint: disable=too-many-instance-attributes
    """Configuration for Azure AI Tool Client.

    Manages authentication, endpoint configuration, and policy settings for the
    Azure AI Tool Client. This class is used internally by the client and should
    not typically be instantiated directly.

    :param credential:
        Azure TokenCredential for authentication.
    :type credential: ~azure.core.credentials.TokenCredential
    """

    def __init__(self, credential: "AsyncTokenCredential"):
        super().__init__()

        self.retry_policy = policies.AsyncRetryPolicy()
        self.logging_policy = policies.NetworkTraceLoggingPolicy()
        self.request_id_policy = policies.RequestIdPolicy()
        self.http_logging_policy = policies.HttpLoggingPolicy()
        self.user_agent_policy = policies.UserAgentPolicy(
            base_user_agent=get_current_app().as_user_agent("FoundryToolClient"))
        self.authentication_policy = policies.AsyncBearerTokenCredentialPolicy(
            credential, "https://ai.azure.com/.default"
        )
        self.redirect_policy = policies.AsyncRedirectPolicy()



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from ._create_response import CreateResponse  # type: ignore
from .projects import Response, ResponseStreamEvent

__all__ = ["CreateResponse", "Response", "ResponseStreamEvent"]  # type: ignore[var-annotated]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""Unit tests for _starlette.py - testing public methods of UserInfoContextMiddleware."""
import pytest
from contextvars import ContextVar
from unittest.mock import AsyncMock, MagicMock

from azure.ai.agentserver.core.tools.client._models import UserInfo


class TestUserInfoContextMiddlewareInstall:
    """Tests for UserInfoContextMiddleware.install class method."""

    def test_install_adds_middleware_to_starlette_app(self):
        """Test install adds middleware to Starlette application."""
        # Import here to avoid requiring starlette when not needed
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        mock_app = MagicMock()

        UserInfoContextMiddleware.install(mock_app)

        mock_app.add_middleware.assert_called_once()
        call_args = mock_app.add_middleware.call_args
        assert call_args[0][0] == UserInfoContextMiddleware

    def test_install_uses_default_context_when_none_provided(self):
        """Test install uses default user context when none is provided."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware
        from azure.ai.agentserver.core.tools.runtime._user import ContextVarUserProvider

        mock_app = MagicMock()

        UserInfoContextMiddleware.install(mock_app)

        call_kwargs = mock_app.add_middleware.call_args[1]
        assert call_kwargs["user_info_var"] is ContextVarUserProvider.default_user_info_context

    def test_install_uses_custom_context(self):
        """Test install uses custom user context when provided."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        mock_app = MagicMock()
        custom_context = ContextVar("custom_context")

        UserInfoContextMiddleware.install(mock_app, user_context=custom_context)

        call_kwargs = mock_app.add_middleware.call_args[1]
        assert call_kwargs["user_info_var"] is custom_context

    def test_install_uses_custom_resolver(self):
        """Test install uses custom user resolver when provided."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        mock_app = MagicMock()

        async def custom_resolver(request):
            return UserInfo(object_id="custom-oid", tenant_id="custom-tid")

        UserInfoContextMiddleware.install(mock_app, user_resolver=custom_resolver)

        call_kwargs = mock_app.add_middleware.call_args[1]
        assert call_kwargs["user_resolver"] is custom_resolver


class TestUserInfoContextMiddlewareDispatch:
    """Tests for UserInfoContextMiddleware.dispatch method."""

    @pytest.mark.asyncio
    async def test_dispatch_sets_user_in_context(self):
        """Test dispatch sets user info in context variable."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        user_context = ContextVar("test_context")
        user_info = UserInfo(object_id="test-oid", tenant_id="test-tid")

        async def mock_resolver(request):
            return user_info

        # Create a simple mock app
        mock_app = AsyncMock()

        middleware = UserInfoContextMiddleware(
            app=mock_app,
            user_info_var=user_context,
            user_resolver=mock_resolver
        )

        mock_request = MagicMock()
        captured_user = None

        async def call_next(request):
            nonlocal captured_user
            captured_user = user_context.get(None)
            return MagicMock()

        await middleware.dispatch(mock_request, call_next)

        assert captured_user is user_info

    @pytest.mark.asyncio
    async def test_dispatch_resets_context_after_request(self):
        """Test dispatch resets context variable after request completes."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        user_context = ContextVar("test_context")
        original_user = UserInfo(object_id="original-oid", tenant_id="original-tid")
        user_context.set(original_user)

        new_user = UserInfo(object_id="new-oid", tenant_id="new-tid")

        async def mock_resolver(request):
            return new_user

        mock_app = AsyncMock()

        middleware = UserInfoContextMiddleware(
            app=mock_app,
            user_info_var=user_context,
            user_resolver=mock_resolver
        )

        mock_request = MagicMock()

        async def call_next(request):
            # During request, should have new_user
            assert user_context.get(None) is new_user
            return MagicMock()

        await middleware.dispatch(mock_request, call_next)

        # After request, context should be reset to original value
        assert user_context.get(None) is original_user

    @pytest.mark.asyncio
    async def test_dispatch_resets_context_on_exception(self):
        """Test dispatch resets context even when call_next raises exception."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        user_context = ContextVar("test_context")
        original_user = UserInfo(object_id="original-oid", tenant_id="original-tid")
        user_context.set(original_user)

        new_user = UserInfo(object_id="new-oid", tenant_id="new-tid")

        async def mock_resolver(request):
            return new_user

        mock_app = AsyncMock()

        middleware = UserInfoContextMiddleware(
            app=mock_app,
            user_info_var=user_context,
            user_resolver=mock_resolver
        )

        mock_request = MagicMock()

        async def call_next(request):
            raise RuntimeError("Request failed")

        with pytest.raises(RuntimeError, match="Request failed"):
            await middleware.dispatch(mock_request, call_next)

        # Context should still be reset to original
        assert user_context.get(None) is original_user

    @pytest.mark.asyncio
    async def test_dispatch_handles_none_user(self):
        """Test dispatch handles None user from resolver."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        user_context = ContextVar("test_context")

        async def mock_resolver(request):
            return None

        mock_app = AsyncMock()

        middleware = UserInfoContextMiddleware(
            app=mock_app,
            user_info_var=user_context,
            user_resolver=mock_resolver
        )

        mock_request = MagicMock()
        captured_user = "not_set"

        async def call_next(request):
            nonlocal captured_user
            captured_user = user_context.get("default")
            return MagicMock()

        await middleware.dispatch(mock_request, call_next)

        assert captured_user is None

    @pytest.mark.asyncio
    async def test_dispatch_calls_resolver_with_request(self):
        """Test dispatch calls user resolver with the request object."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        user_context = ContextVar("test_context")
        captured_request = None

        async def mock_resolver(request):
            nonlocal captured_request
            captured_request = request
            return UserInfo(object_id="oid", tenant_id="tid")

        mock_app = AsyncMock()

        middleware = UserInfoContextMiddleware(
            app=mock_app,
            user_info_var=user_context,
            user_resolver=mock_resolver
        )

        mock_request = MagicMock()
        mock_request.url = "https://test.com/api"

        async def call_next(request):
            return MagicMock()

        await middleware.dispatch(mock_request, call_next)

        assert captured_request is mock_request


class TestUserInfoContextMiddlewareDefaultResolver:
    """Tests for UserInfoContextMiddleware default resolver."""

    @pytest.mark.asyncio
    async def test_default_resolver_extracts_user_from_headers(self):
        """Test default resolver extracts user info from request headers."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        mock_request = MagicMock()
        mock_request.headers = {
            "x-aml-oid": "header-object-id",
            "x-aml-tid": "header-tenant-id"
        }

        result = await UserInfoContextMiddleware._default_user_resolver(mock_request)

        assert result is not None
        assert result.object_id == "header-object-id"
        assert result.tenant_id == "header-tenant-id"

    @pytest.mark.asyncio
    async def test_default_resolver_returns_none_when_headers_missing(self):
        """Test default resolver returns None when required headers are missing."""
        from azure.ai.agentserver.core.tools.runtime._starlette import UserInfoContextMiddleware

        mock_request = MagicMock()
        mock_request.headers = {}

        result = await UserInfoContextMiddleware._default_user_resolver(mock_request)

        assert result is None



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
from typing import AsyncContextManager, ClassVar, Optional

from azure.ai.agentserver.core.tools import FoundryToolRuntime


class AgentServerContext(AsyncContextManager["AgentServerContext"]):
    _INSTANCE: ClassVar[Optional["AgentServerContext"]] = None

    def __init__(self, tool_runtime: FoundryToolRuntime):
        self._tool_runtime = tool_runtime

        self.__class__._INSTANCE = self

    @classmethod
    def get(cls) -> "AgentServerContext":
        if cls._INSTANCE is None:
            raise ValueError("AgentServerContext has not been initialized.")
        return cls._INSTANCE

    @property
    def tools(self) -> FoundryToolRuntime:
        return self._tool_runtime

    async def __aenter__(self) -> "AgentServerContext":
        await self._tool_runtime.__aenter__()
        return self

    async def __aexit__(self, exc_type, exc_value, traceback) -> None:
        await self._tool_runtime.__aexit__(exc_type, exc_value, traceback)
